@@ -23,8 +23,13 @@
 #ifndef ___BLADEDLL_H_INCLUDED___
 #define ___BLADEDLL_H_INCLUDED___
 
+#ifdef __GNUC__
+#define ATTRIBUTE_PACKED	__attribute__((packed))
+#else
+#define ATTRIBUTE_PACKED
 #pragma pack(push)
 #pragma pack(1)
+#endif
 
 #ifdef	__cplusplus
 extern "C" {
@@ -190,7 +195,7 @@ typedef struct	{
 
 	} format;
 		
-} BE_CONFIG, *PBE_CONFIG;
+} BE_CONFIG, *PBE_CONFIG ATTRIBUTE_PACKED;
 
 
 typedef struct	{
@@ -222,7 +227,7 @@ typedef struct	{
 	BYTE	btReserved[125];
 
 
-} BE_VERSION, *PBE_VERSION;			
+} BE_VERSION, *PBE_VERSION ATTRIBUTE_PACKED;
 
 #ifndef _BLADEDLL
 
@@ -264,7 +269,9 @@ __declspec(dllexport) BE_ERR	beWriteInfoTag( HBE_STREAM hbeStream, LPCSTR lpszFi
 
 #endif
 
+#ifndef __GNUC__
 #pragma pack(pop)
+#endif
 
 #ifdef	__cplusplus
 }
