@@ -41,7 +41,7 @@
 #define PACKAGE "lame"
 
 /* Version number of package */
-#define VERSION "3.96"
+#define VERSION "3.97"
 
 /* Define if compiler has function prototypes */
 #define PROTOTYPES 1
@@ -58,7 +58,7 @@
 #define HAVE_STRCHR
 #define HAVE_MEMCPY
 
-#if defined(_MSC_VER)
+#if defined(_MSC_VER) || defined(__BORLANDC__)
 #pragma warning( disable : 4305 )
 	typedef __int8  int8_t;
 	typedef __int16 int16_t;
@@ -73,6 +73,7 @@
 	typedef float  float32_t;
 	typedef double float64_t;
 #elif defined (__GNUC__)
+#define __int8_t_defined
 #define uint8_t unsigned char
 #define uint16_t unsigned short
 #define uint32_t unsigned int
@@ -106,3 +107,14 @@ void acm_Free( void * mem);
 
 #define LAME_LIBRARY_BUILD
 
+
+#ifdef HAVE_NASM
+    #if (defined(__ICL) && (__ICL >= 450))
+        #define HAVE_XMMINTRIN_H
+    #elif defined(_MSC_VER)
+        #include <malloc.h>
+        #ifdef _mm_malloc
+            #define HAVE_XMMINTRIN_H
+        #endif
+    #endif
+#endif
