@@ -2,7 +2,7 @@
  *      Command line frontend program
  *
  *      Copyright (c) 1999 Mark Taylor
- *                    2000 Takehiro TOMIANGA
+ *                    2000 Takehiro TOMINAGA
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: main.c,v 1.85 2003/12/21 18:41:33 bouvigne Exp $ */
+/* $Id: main.c,v 1.92.2.1 2005/12/18 18:49:29 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -71,7 +71,6 @@ char   *strchr(), *strrchr();
 #include "get_audio.h"
 #include "portableio.h"
 #include "timestatus.h"
-#include "VbrTag.h"
 
 /* PLL 14/04/2000 */
 #if macintosh
@@ -308,7 +307,8 @@ lame_decoder(lame_global_flags * gfp, FILE * outf, int skip, char *inPath,
                             tmp_num_channels, 16);
     fclose(outf);
 
-    decoder_progress_finish(gfp);
+    if (silent <= 0)
+	decoder_progress_finish(gfp);
     return 0;
 }
 
@@ -452,6 +452,7 @@ lame_encoder(lame_global_flags * gf, FILE * outf, int nogap, char *inPath,
 #ifdef BRHIST
         brhist_jump_back();
 #endif
+        frames = lame_get_frameNum(gf);
         timestatus(lame_get_out_samplerate(gf),
                    frames, lame_get_totalframes(gf), lame_get_framesize(gf));
 #ifdef BRHIST
@@ -572,6 +573,61 @@ void parse_nogap_filenames(int nogapout, char *inPath, char *outPath, char *outd
 }
 
 
+  
+
+void print_lame_tag_leading_info(lame_global_flags *gf) {
+    if(lame_get_bWriteVbrTag(gf))
+      printf("Writing LAME Tag...");
+}
+
+void print_trailing_info(lame_global_flags *gf)
+{
+    if (lame_get_bWriteVbrTag(gf))
+      printf("done\n");
+
+    if (lame_get_findReplayGain(gf) ) {
+      int RadioGain = lame_get_RadioGain(gf);
+      printf("ReplayGain: %s%.1fdB\n", RadioGain > 0 ? "+" : "", ((float)RadioGain) / 10.0);
+      if (RadioGain > 0x1FE || RadioGain < -0x1FE) 
+	printf("WARNING: ReplayGain exceeds the -51dB to +51dB range. Such a result is too\n"
+	       "         high to be stored in the header.\n" );
+    }
+	
+    /* if (the user requested printing info about clipping) and (decoding
+       on the fly has actually been performed) */
+    if(print_clipping_info && lame_get_decode_on_the_fly(gf)) {
+      float noclipGainChange = (float)lame_get_noclipGainChange(gf) / 10.0;
+      float noclipScale = lame_get_noclipScale(gf);
+
+      if (noclipGainChange > 0.0) { /* clipping occurs */
+	printf("WARNING: clipping occurs at the current gain. Set your decoder to decrease\n"
+	       "         the  gain  by  at least %.1fdB or encode again ", noclipGainChange); 
+                                                                            
+	/* advice the user on the scale factor */
+	if (noclipScale > 0) {
+	  printf("using  --scale %.2f\n", noclipScale);
+	  printf("         or less (the value under --scale is approximate).\n" );
+	}
+	else {
+          /* the user specified his own scale factor. We could suggest 
+           * the scale factor of (32767.0/gfp->PeakSample)*(gfp->scale)
+           * but it's usually very inaccurate. So we'd rather advice him to 
+           * disable scaling first and see our suggestion on the scale factor then. */
+	  printf("using --scale <arg>\n"
+	         "         (For   a   suggestion  on  the  optimal  value  of  <arg>  encode\n"
+	         "         with  --scale 1  first)\n" );
+	}
+		    
+      }
+      else { /* no clipping */
+	if (noclipGainChange > -0.1)
+	  printf("\nThe waveform does not clip and is less than 0.1dB away from full scale.\n" );
+	else                                                                                        
+	  printf("\nThe waveform does not clip and is at least %.1fdB away from full scale.\n", -noclipGainChange);
+      }                                                                                             
+    }
+  
+}
 
 
 
@@ -632,7 +688,7 @@ main(int argc, char **argv)
         return 1;
     }
     if (argc <= 1) {
-        usage(gf, stderr, argv[0]); /* no command-line args, print usage, exit  */
+        usage(stderr, argv[0]); /* no command-line args, print usage, exit  */
         return 1;
     }
 
@@ -682,7 +738,11 @@ main(int argc, char **argv)
         return i;
     }
 
-    if (silent > 0 || lame_get_VBR(gf) == vbr_off) {
+    if (silent > 0 
+#ifndef RH_HIST   
+    || lame_get_VBR(gf) == vbr_off
+#endif
+    ) {
         brhist = 0;     /* turn off VBR histogram */
     }
 
@@ -709,13 +769,16 @@ main(int argc, char **argv)
                     outf = init_files(gf, nogap_inPath[i], outPath);
                 }
                 brhist_init_package(gf);
+                lame_set_nogap_total(gf, max_nogap);
+                lame_set_nogap_currentindex(gf, i);
                 ret =
                     lame_encoder(gf, outf, use_flush_nogap, nogap_inPath[i],
                                  outPath);	
                 
-                if (silent<=0) ReportLameTagProgress(gf,1);
+                if (silent<=0) print_lame_tag_leading_info(gf);
                 lame_mp3_tags_fid(gf, outf); /* add VBR tags to mp3 file */
-                if (silent<=0) ReportLameTagProgress(gf,0);
+		
+                if (silent<=0) print_trailing_info(gf);
                 
                 fclose(outf); /* close the output file */
                 close_infile(); /* close the input file */
@@ -735,9 +798,10 @@ main(int argc, char **argv)
             brhist_init_package(gf);
             ret = lame_encoder(gf, outf, 0, inPath, outPath);
             
-            if (silent<=0) ReportLameTagProgress(gf,1);
+            if (silent<=0) print_lame_tag_leading_info(gf);
             lame_mp3_tags_fid(gf, outf); /* add VBR tags to mp3 file */
-            if (silent<=0) ReportLameTagProgress(gf,0);
+	    
+            if (silent<=0) print_trailing_info(gf);
             
             fclose(outf); /* close the output file */
             close_infile(); /* close the input file */
@@ -747,3 +811,5 @@ main(int argc, char **argv)
     return ret;
 }
 
+
+
