@@ -1,7 +1,8 @@
 /*
  *	MP3 bitstream Output interface for LAME
  *
- *	Copyright (c) 1999 Takehiro TOMINAGA
+ *	Copyright (c) 1999-2000 Mark Taylor
+ *	Copyright (c) 1999-2002 Takehiro Tominaga
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -18,7 +19,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: bitstream.c,v 1.66 2004/03/23 01:57:17 olcios Exp $
+ * $Id: bitstream.c,v 1.69.2.1 2005/11/20 14:08:24 bouvigne Exp $
  */
 
 
@@ -81,7 +82,7 @@ int getframebits(const lame_global_flags * gfp)
 
 
 
-void putheader_bits(lame_internal_flags *gfc,int w_ptr)
+void putheader_bits(lame_internal_flags *gfc)
 {
     Bit_stream_struc *bs;
     bs = &gfc->bs;
@@ -116,7 +117,7 @@ putbits2(lame_internal_flags *gfc, int val, int j)
 	    assert(bs->buf_byte_idx < BUFFER_SIZE);
 	    assert(gfc->header[gfc->w_ptr].write_timing >= bs->totbit);
 	    if (gfc->header[gfc->w_ptr].write_timing == bs->totbit) {
-	      putheader_bits(gfc,gfc->w_ptr);
+	      putheader_bits(gfc);
 	    }
 	    bs->buf[bs->buf_byte_idx] = 0;
 	}
@@ -442,7 +443,7 @@ huffman_coder_count1(lame_internal_flags *gfc, gr_info *gi)
 #endif
 
     int *ix = &gi->l3_enc[gi->big_values];
-    FLOAT8 *xr = &gi->xr[gi->big_values];
+    FLOAT *xr = &gi->xr[gi->big_values];
     assert(gi->count1table_select < 2);
 
     for (i = (gi->count1 - gi->big_values) / 4; i > 0; --i) {
@@ -822,17 +823,17 @@ flush_bitstream(lame_global_flags *gfp)
 
 
   /* save the ReplayGain value */
-  if (gfp->findReplayGain) {
+  if (gfc->findReplayGain) {
     FLOAT RadioGain = (FLOAT) GetTitleGain(gfc->rgdata);
     assert(RadioGain != GAIN_NOT_ENOUGH_SAMPLES); 
-    gfp->internal_flags->RadioGain = (int) floor( RadioGain * 10.0 + 0.5 ); /* round to nearest */
+    gfc->RadioGain = (int) floor( RadioGain * 10.0 + 0.5 ); /* round to nearest */
   }
 
   /* find the gain and scale change required for no clipping */
   if (gfc->findPeakSample) {
     gfc->noclipGainChange = (int) ceil(log10(gfc->PeakSample / 32767.0) *20.0*10.0);  /* round up */
     
-    if (gfc->noclipGainChange > 0.0) { /* clipping occurs */
+    if (gfc->noclipGainChange > 0) { /* clipping occurs */
       if (gfp->scale == 1.0 || gfp->scale == 0.0) 
 	gfc->noclipScale = floor( (32767.0 / gfc->PeakSample) * 100.0 ) / 100.0;  /* round down */
       else
@@ -969,7 +970,7 @@ int copy_buffer(lame_internal_flags *gfc,unsigned char *buffer,int size,int mp3d
         UpdateMusicCRC(&gfc->nMusicCRC,buffer,minimum);
 
 #ifdef DECODE_ON_THE_FLY 
-        if (gfc->gfp->decode_on_the_fly) {  /* decode the frame */
+        if (gfc->decode_on_the_fly) {  /* decode the frame */
           sample_t pcm_buf[2][1152];
 	  int mp3_in = minimum;
           int samples_out = -1;
@@ -1016,13 +1017,13 @@ int copy_buffer(lame_internal_flags *gfc,unsigned char *buffer,int size,int mp3d
                   }
               }
 
-              if (gfc->gfp->findReplayGain)
+              if (gfc->findReplayGain)
                 if (AnalyzeSamples(gfc->rgdata, pcm_buf[0], pcm_buf[1], samples_out, gfc->channels_out) == GAIN_ANALYSIS_ERROR)
                    return -6;
 		   
             } /* if (samples_out>0) */
           } /* while (samples_out!=0) */
-        } /* if (gfp->decode_on_the_fly) */ 
+        } /* if (gfc->decode_on_the_fly) */ 
 #endif
   
     } /* if (mp3data) */
