@@ -28,7 +28,7 @@
  * NOTE: See http://id3.org/ for more information about ID3 tag formats.
  */
 
-/* $Id: id3tag.c,v 1.22 2003/06/01 16:12:44 bouvigne Exp $ */
+/* $Id: id3tag.c,v 1.23.4.1 2006/01/21 16:29:37 takehiro Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -392,7 +392,7 @@ id3tag_write_v2(lame_global_flags *gfp)
             size_t encoder_length;
             char year[5];
             size_t year_length;
-            char track[3];
+            char track[4];
             size_t track_length;
             char genre[6];
             size_t genre_length;
@@ -402,8 +402,18 @@ id3tag_write_v2(lame_global_flags *gfp)
             unsigned int index;
             /* calulate size of tag starting with 10-byte tag header */
             tag_size = 10;
+#if defined(__hpux) || defined(__svr4__) || defined(M_UNIX) || defined(_AIX)
+            encoder_length = sprintf(encoder,
+                            "LAME v%s", get_lame_short_version());
+#else
+#if defined(__sun__)
+            (void) sprintf(encoder, "LAME v%s", get_lame_short_version());
+            encoder_length = strlen(encoder);
+#else
             encoder_length = snprintf(encoder, sizeof(encoder),
                             "LAME v%s", get_lame_short_version());
+#endif
+#endif
             tag_size += 11 + encoder_length;
             if (title_length) {
                 /* add 10-byte frame header, 1 encoding descriptor byte ... */
