@@ -1,7 +1,12 @@
 /*
  *	psymodel.c
  *
- *	Copyright (c) 1999 Mark Taylor
+ *	Copyright (c) 1999-2000 Mark Taylor
+ *	Copyright (c) 2001-2002 Naoki Shibata
+ *	Copyright (c) 2000-2003 Takehiro Tominaga
+ *	Copyright (c) 2000-2005 Robert Hegemann
+ *	Copyright (c) 2000-2005 Gabriel Bouvigne
+ *	Copyright (c) 2000-2005 Alexander Leidinger
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,7 +24,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: psymodel.c,v 1.135.2.2 2004/06/27 19:32:34 robert Exp $ */
+/* $Id: psymodel.c,v 1.142.2.2 2006/08/18 18:22:20 bouvigne Exp $ */
 
 
 /*
@@ -368,7 +373,7 @@ msfix1(
     )
 {
     int sb, sblock;
-    FLOAT8 rside,rmid,mld;
+    FLOAT rside,rmid,mld;
     for ( sb = 0; sb < SBMAX_l; sb++ ) {
 	/* use this fix if L & R masking differs by 2db or less */
 	/* if db = 10*log10(x2/x1) < 2 */
@@ -431,7 +436,7 @@ ns_msfix(
 	thmS = Max(gfc->thm[3].l[sb],ath);
 
 	if (thmLR*msfix < thmM+thmS) {
-	    FLOAT8 f = thmLR * msfix2 / (thmM+thmS);
+	    FLOAT f = thmLR * msfix2 / (thmM+thmS);
 	    thmM *= f;
 	    thmS *= f;
 	}
@@ -439,10 +444,10 @@ ns_msfix(
 	gfc->thm[3].l[sb] = Min(thmS,gfc->thm[3].l[sb]);
     }
 
-    athlower *= (BLKSIZE_s / BLKSIZE);
+    athlower *= ((FLOAT)BLKSIZE_s / BLKSIZE);
     for ( sb = 0; sb < SBMAX_s; sb++ ) {
 	for ( sblock = 0; sblock < 3; sblock++ ) {
-	    FLOAT8 thmLR,thmM,thmS,ath;
+	    FLOAT thmLR,thmM,thmS,ath;
 	    ath  = (gfc->ATH->cb[gfc->bm_s[sb]])*athlower;
 	    thmLR = Min(Max(gfc->thm[0].s[sb][sblock],ath),
 			Max(gfc->thm[1].s[sb][sblock],ath));
@@ -463,12 +468,12 @@ ns_msfix(
 /* longblock threshold calculation (part 2) */
 static void convert_partition2scalefac_l(
     lame_internal_flags *gfc,
-    FLOAT8 *eb,
-    FLOAT8 *thr,
+    FLOAT *eb,
+    FLOAT *thr,
     int chn
     )
 {
-    FLOAT8 enn, thmm;
+    FLOAT enn, thmm;
     int sb, b;
     enn = thmm = 0.0;
     sb = b = 0;
@@ -503,15 +508,15 @@ static void
 compute_masking_s(
     lame_internal_flags *gfc,
     FLOAT (*fftenergy_s)[HBLKSIZE_s],
-    FLOAT8 *eb,
-    FLOAT8 *thr,
+    FLOAT *eb,
+    FLOAT *thr,
     int chn,
     int sblock,
-    FLOAT8 athlower
+    FLOAT athlower
     )
 {
     int j, b;
-    athlower *= (BLKSIZE_s / BLKSIZE);
+    athlower *= ((FLOAT)BLKSIZE_s / BLKSIZE);
     for (j = b = 0; b < gfc->npart_s; b++) {
 	FLOAT ecb = fftenergy_s[sblock][j++];
 	int kk = gfc->numlines_s[b];
@@ -521,7 +526,7 @@ compute_masking_s(
     }
     for (j = b = 0; b < gfc->npart_s; b++) {
 	int kk = gfc->s3ind_s[b][0];
-	FLOAT8 ecb = gfc->s3_ss[j++] * eb[kk++];
+	FLOAT ecb = gfc->s3_ss[j++] * eb[kk++];
 	while (kk <= gfc->s3ind_s[b][1])
 	    ecb += gfc->s3_ss[j++] * eb[kk++];
 
@@ -529,7 +534,9 @@ compute_masking_s(
 	if (gfc->blocktype_old[chn & 1] == SHORT_TYPE ) {
 	    thr[b] = Min(thr[b], rpelev2_s * gfc->nb_s2[chn][b]);
 	}
-	thr[b] = Max( thr[b], gfc->ATH->cb[gfc->bm_s[b]] * athlower );
+	thr[b] = Max( thr[b],
+                  Min(gfc->ATH->cb[gfc->bm_s[b]] * athlower,
+                      thr[b] * 2) );
 	gfc->nb_s2[chn][b] = gfc->nb_s1[chn][b];
 	gfc->nb_s1[chn][b] = ecb;
     assert( thr[b] >= 0 );
@@ -584,6 +591,57 @@ block_type_set(
     }
 }
 
+static void 
+determine_block_type( lame_global_flags * gfp, FLOAT fftenergy_s[3][HBLKSIZE_s], int uselongblock[],
+    int chn, int gr_out, FLOAT* pe )
+{
+    lame_internal_flags* gfc = gfp->internal_flags;
+    int j;
+	/*************************************************************** 
+	 * determine the block type (window type) based on L & R channels
+	 ***************************************************************/
+	/* compute PE for all 4 channels */
+	    FLOAT mn,mx,ma=0,mb=0,mc=0;
+            for ( j = HBLKSIZE_s/2; j < HBLKSIZE_s; j ++) {
+		ma += fftenergy_s[0][j];
+		mb += fftenergy_s[1][j];
+		mc += fftenergy_s[2][j];
+	    }
+	    mn = Min(ma,mb);
+	    mn = Min(mn,mc);
+	    mx = Max(ma,mb);
+	    mx = Max(mx,mc);
+#if defined(HAVE_GTK)
+	    if (gfp->analysis) {
+		gfc->pinfo->ers[gr_out][chn]=gfc->ers_save[chn];
+		gfc->ers_save[chn]=(mx/(1e-12+mn));
+	    }
+#endif
+	    /* bit allocation is based on pe.  */
+	    if (mx>mn) {
+		FLOAT tmp = FAST_LOG_X(mx/(1e-12+mn), 400.0);
+		if (tmp > *pe) *pe = tmp;
+	    }
+
+	    /* block type is based just on L or R channel */      
+	    if (chn<2) {
+		uselongblock[chn] = 1;
+
+		/* tuned for t1.wav.  doesnt effect most other samples */
+		if (*pe > 3000) 
+		    uselongblock[chn]=0;
+	
+		if ( mx > 30*mn ) 
+		{/* big surge of energy - always use short blocks */
+		    uselongblock[chn] = 0;
+		}
+		else if ((mx > 10*mn) && (*pe > 1000))
+		{/* medium surge, medium pe - use short blocks */
+		    uselongblock[chn] = 0;
+		}
+	    }
+}
+
 int L3psycho_anal( lame_global_flags * gfp,
                     const sample_t *buffer[2], int gr_out, 
                     FLOAT *ms_ratio,
@@ -603,12 +661,12 @@ int L3psycho_anal( lame_global_flags * gfp,
     FLOAT fftenergy_s[3][HBLKSIZE_s];
 
     /* convolution   */
-    FLOAT8 eb[CBANDS+1];
-    FLOAT8 cb[CBANDS];
-    FLOAT8 thr[CBANDS+1];
+    FLOAT eb[CBANDS+1];
+    FLOAT cb[CBANDS];
+    FLOAT thr[CBANDS+1];
 
     /* ratios    */
-    FLOAT8 ms_ratio_l=0, ms_ratio_s=0;
+    FLOAT ms_ratio_l=0, ms_ratio_s=0;
 
     /* block type  */
     int blocktype[2],uselongblock[2];
@@ -769,7 +827,7 @@ int L3psycho_anal( lame_global_flags * gfp,
 	b = 0;
 	for (j = 0; j < gfc->cw_upper_index
 		 && gfc->numlines_l[b] && b < gfc->npart_l; ) {
-	    FLOAT8 ebb, cbb;
+	    FLOAT ebb, cbb;
 
 	    ebb = NON_LINEAR_SCALE_ITEM(fftenergy[j]);
 	    cbb = NON_LINEAR_SCALE_ITEM(fftenergy[j] * gfc->cw[j]);
@@ -787,7 +845,7 @@ int L3psycho_anal( lame_global_flags * gfp,
 	}
 
 	for (; b < gfc->npart_l; b++ ) {
-	    FLOAT8 ebb = NON_LINEAR_SCALE_ITEM(fftenergy[j++]);
+	    FLOAT ebb = NON_LINEAR_SCALE_ITEM(fftenergy[j++]);
 	    assert(gfc->numlines_l[b]);
 	    for (i = gfc->numlines_l[b] - 1; i > 0; i--) {
 		ebb += NON_LINEAR_SCALE_ITEM(fftenergy[j++]);
@@ -805,7 +863,7 @@ int L3psycho_anal( lame_global_flags * gfp,
 	gfc->pe[chn] = 0;
 	k = 0;
 	for ( b = 0;b < gfc->npart_l; b++ ) {
-	    FLOAT8 tbb,ecb,ctb;
+	    FLOAT tbb,ecb,ctb;
 	    int kk;
 	    ecb = ctb = 0.;
 	    for (kk = gfc->s3ind[b][0]; kk <= gfc->s3ind[b][1]; kk++ ) {
@@ -841,8 +899,8 @@ int L3psycho_anal( lame_global_flags * gfp,
 	    tbb = Min(gfc->minval[b], tbb);
 	    /* stabilize tonality estimation */
 	    if (gfc->PSY->tonalityPatch && b > 5) {
-		FLOAT8 const x = 1.8699422;
-		FLOAT8 w = gfc->PSY->prvTonRed[b/2] * x;
+		FLOAT const x = 1.8699422;
+		FLOAT w = gfc->PSY->prvTonRed[b/2] * x;
 		if (tbb > w) 
 		    tbb = w;
 		gfc->PSY->prvTonRed[b] = tbb;
@@ -877,57 +935,14 @@ int L3psycho_anal( lame_global_flags * gfp,
 		gfc->pe[chn] -= gfc->numlines_l[b] * FAST_LOG(ecb / eb[b]);
 	}
 
-	/*************************************************************** 
-	 * determine the block type (window type) based on L & R channels
-	 ***************************************************************/
-	{  /* compute PE for all 4 channels */
-	    FLOAT mn,mx,ma=0,mb=0,mc=0;
-            for ( j = HBLKSIZE_s/2; j < HBLKSIZE_s; j ++) {
-		ma += fftenergy_s[0][j];
-		mb += fftenergy_s[1][j];
-		mc += fftenergy_s[2][j];
-	    }
-	    mn = Min(ma,mb);
-	    mn = Min(mn,mc);
-	    mx = Max(ma,mb);
-	    mx = Max(mx,mc);
-#if defined(HAVE_GTK)
-	    if (gfp->analysis) {
-		gfc->pinfo->ers[gr_out][chn]=gfc->ers_save[chn];
-		gfc->ers_save[chn]=(mx/(1e-12+mn));
-	    }
-#endif
-	    /* bit allocation is based on pe.  */
-	    if (mx>mn) {
-		FLOAT8 tmp = FAST_LOG_X(mx/(1e-12+mn), 400.0);
-		if (tmp>gfc->pe[chn]) gfc->pe[chn]=tmp;
-	    }
-
-	    /* block type is based just on L or R channel */      
-	    if (chn<2) {
-		uselongblock[chn] = 1;
-
-		/* tuned for t1.wav.  doesnt effect most other samples */
-		if (gfc->pe[chn] > 3000) 
-		    uselongblock[chn]=0;
-	
-		if ( mx > 30*mn ) 
-		{/* big surge of energy - always use short blocks */
-		    uselongblock[chn] = 0;
-		}
-		else if ((mx > 10*mn) && (gfc->pe[chn] > 1000))
-		{/* medium surge, medium pe - use short blocks */
-		    uselongblock[chn] = 0;
-		}
-	    }
-	}
+    determine_block_type( gfp, fftenergy_s, uselongblock, chn, gr_out, &gfc->pe[chn] );
 
 	/* compute masking thresholds for long blocks */
 	convert_partition2scalefac_l(gfc, eb, thr, chn);
 
 	/* compute masking thresholds for short blocks */
 	for (sblock = 0; sblock < 3; sblock++) {
-	    FLOAT8 enn, thmm;
+	    FLOAT enn, thmm;
 	    compute_masking_s(gfc, fftenergy_s, eb, thr, chn,
 			      sblock,gfp->ATHlower*gfc->ATH->adjust);
 	    b = -1;
@@ -957,7 +972,7 @@ int L3psycho_anal( lame_global_flags * gfp,
 	calc_interchannel_masking(gfp, gfp->interChRatio);
 
     if (gfp->mode == JOINT_STEREO) {
-	FLOAT8 db,x1,x2,sidetot=0,tot=0;
+	FLOAT db,x1,x2,sidetot=0,tot=0;
 	msfix1(gfc);
 	if (gfp->msfix != 0.0)
 	    ns_msfix(gfc, gfp->msfix, gfp->ATHlower*gfc->ATH->adjust);
@@ -1033,9 +1048,9 @@ int L3psycho_anal( lame_global_flags * gfp,
 #define I2LIMIT 23  /* as in if(i>24) -> changed 23 */ 
 #define MLIMIT  15  /* as in if(m<15) */ 
 
-static FLOAT8 ma_max_i1;
-static FLOAT8 ma_max_i2;
-static FLOAT8 ma_max_m;
+static FLOAT ma_max_i1;
+static FLOAT ma_max_i2;
+static FLOAT ma_max_m;
 
 
 
@@ -1052,22 +1067,22 @@ static void init_mask_add_max_values(void)
 
 
 /* addition of simultaneous masking   Naoki Shibata 2000/7 */
-inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_flags * const gfc)
+inline static FLOAT mask_add(FLOAT m1,FLOAT m2,int k,int b, lame_internal_flags * const gfc)
 {
-  static const FLOAT8 table1[] = {
+  static const FLOAT table1[] = {
     3.3246 *3.3246 ,3.23837*3.23837,3.15437*3.15437,3.00412*3.00412,2.86103*2.86103,2.65407*2.65407,2.46209*2.46209,2.284  *2.284  ,
     2.11879*2.11879,1.96552*1.96552,1.82335*1.82335,1.69146*1.69146,1.56911*1.56911,1.46658*1.46658,1.37074*1.37074,1.31036*1.31036,
     1.25264*1.25264,1.20648*1.20648,1.16203*1.16203,1.12765*1.12765,1.09428*1.09428,1.0659 *1.0659 ,1.03826*1.03826,1.01895*1.01895,
     1
   };
 
-  static const FLOAT8 table2[] = {
+  static const FLOAT table2[] = {
     1.33352*1.33352,1.35879*1.35879,1.38454*1.38454,1.39497*1.39497,1.40548*1.40548,1.3537 *1.3537 ,1.30382*1.30382,1.22321*1.22321,
     1.14758*1.14758,
     1
   };
 
-  static const FLOAT8 table3[] = {
+  static const FLOAT table3[] = {
     2.35364*2.35364,2.29259*2.29259,2.23313*2.23313,2.12675*2.12675,2.02545*2.02545,1.87894*1.87894,1.74303*1.74303,1.61695*1.61695,
     1.49999*1.49999,1.39148*1.39148,1.29083*1.29083,1.19746*1.19746,1.11084*1.11084,1.03826*1.03826
   };
@@ -1078,9 +1093,10 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
 
 
   if (m2 > m1) {
-      if (m2 >= (m1*ma_max_i2))
-          return (m1+m2);
-      ratio = m2/m1;
+      if (m2 < (m1*ma_max_i2))
+        ratio = m2/m1;
+      else
+        return (m1+m2);
   } else {
       if (m1 >= (m2*ma_max_i2))
           return (m1+m2);
@@ -1122,7 +1138,7 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
       /* 3% of the total */
       /* Originally if (m > 0) { */
       if (m1 > m2) {
-	  FLOAT8 f, r;
+	  FLOAT f, r;
 
 	  f = 1.0;
 	  if (i <= 13) f = table3[i];
@@ -1143,7 +1159,7 @@ inline static FLOAT8 mask_add(FLOAT8 m1,FLOAT8 m2,int k,int b, lame_internal_fla
 
 
 
-static inline FLOAT8 NS_INTERP(FLOAT8 x, FLOAT8 y, FLOAT8 r)
+static inline FLOAT NS_INTERP(FLOAT x, FLOAT y, FLOAT r)
 {
     /* was pow((x),(r))*pow((y),1-(r))*/
     if(r==1.0)
@@ -1159,8 +1175,8 @@ static inline FLOAT8 NS_INTERP(FLOAT8 x, FLOAT8 y, FLOAT8 r)
 
 static void nsPsy2dataRead(
     FILE *fp,
-    FLOAT8 *eb2,
-    FLOAT8 *eb,
+    FLOAT *eb2,
+    FLOAT *eb,
     int chn,
     int npart_l
     )
@@ -1193,8 +1209,8 @@ pecalc_s(
     FLOAT masking_lower
     )
 {
-    FLOAT8 pe_s;
-    const static FLOAT8 regcoef_s[] = {
+    FLOAT pe_s;
+    const static FLOAT regcoef_s[] = {
 	11.8, /* these values are tuned only for 44.1kHz... */
 	13.6,
 	17.2,
@@ -1237,8 +1253,8 @@ pecalc_l(
     FLOAT masking_lower
     )
 {
-    FLOAT8 pe_l;
-    const static FLOAT8 regcoef_l[] = {
+    FLOAT pe_l;
+    const static FLOAT regcoef_l[] = {
 	6.8, /* these values are tuned only for 44.1kHz... */
 	5.8,
 	5.8,
@@ -1367,16 +1383,17 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	FLOAT (*wsamp_l)[BLKSIZE];
 	FLOAT (*wsamp_s)[3][BLKSIZE_s];
 	FLOAT en_subshort[12];
+    FLOAT   en_short[4] = { 0 };
 	FLOAT attack_intensity[12];
 	int ns_uselongblock = 1;
 	FLOAT attackThreshold;
-	FLOAT8 max[CBANDS],avg[CBANDS];
+	FLOAT max[CBANDS],avg[CBANDS];
 	int ns_attacks[4] = {0};
 	FLOAT fftenergy[HBLKSIZE];
 	FLOAT fftenergy_s[3][HBLKSIZE_s];
 	/* convolution   */
-	FLOAT8 eb[CBANDS+1],eb2[CBANDS];
-	FLOAT8 thr[CBANDS+1];
+	FLOAT eb[CBANDS+1],eb2[CBANDS];
+	FLOAT thr[CBANDS+1];
 
 
     /*This is the masking table:
@@ -1385,7 +1402,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
       After additive masking computation, 8dB are added, so
       final values are going from 8dB to 17.3dB
     */
-    static const FLOAT8 tab[] = {
+    static const FLOAT tab[] = {
         1.0/*pow(10, -0)*/,
         0.79433/*pow(10, -0.1)*/,
         0.63096/*pow(10, -0.2)*/,
@@ -1416,6 +1433,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    en_subshort[i] = gfc->nsPsy.last_en_subshort[chn][i+6];
 	    attack_intensity[i]
 		= en_subshort[i] / gfc->nsPsy.last_en_subshort[chn][i+4];
+        en_short[0] += en_subshort[i];
 	}
 
 	if (chn == 2) {
@@ -1428,21 +1446,23 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    }
 	}
 	{
-	    FLOAT *pf = ns_hpfsmpl[chn & 1];
-	    for (i=0;i<9;i++) {
-		FLOAT *pfe = pf + 576/9, p = 1.;
-		for (; pf < pfe; pf++)
-		    if (p < fabs(*pf))
-			p = fabs(*pf);
+        FLOAT const *pf = ns_hpfsmpl[chn & 1];
+        for (i = 0; i < 9; i++) {
+            FLOAT const *const pfe = pf + 576 / 9;
+            FLOAT   p = 1.;
+            for (; pf < pfe; pf++)
+                if (p < fabs(*pf))
+                    p = fabs(*pf);
 
-		gfc->nsPsy.last_en_subshort[chn][i] = en_subshort[i+3] = p;
-		if (p > en_subshort[i+3-2])
-		    p = p / en_subshort[i+3-2];
-		else if (en_subshort[i+3-2] > p*10.0)
-		    p = en_subshort[i+3-2] / (p*10.0);
-		else
-		    p = 0.0;
-		attack_intensity[i+3] = p;
+            gfc->nsPsy.last_en_subshort[chn][i] = en_subshort[i + 3] = p;
+            en_short[1 + i / 3] += p;
+            if (p > en_subshort[i + 3 - 2])
+                p = p / en_subshort[i + 3 - 2];
+            else if (en_subshort[i + 3 - 2] > p * 10.0)
+                p = en_subshort[i + 3 - 2] / (p * 10.0);
+            else
+                p = 0.0;
+            attack_intensity[i + 3] = p;
 	    }
 	}
 #if defined(HAVE_GTK)
@@ -1455,29 +1475,50 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    gfc->ers_save[chn] = x;
 	}
 #endif
-	/* compare energies between sub-shortblocks */
-	attackThreshold = (chn == 3)
-	    ? gfc->nsPsy.attackthre_s : gfc->nsPsy.attackthre;
-	for (i=0;i<12;i++) 
-	    if (!ns_attacks[i/3] && attack_intensity[i] > attackThreshold)
-		ns_attacks[i/3] = (i % 3)+1;
+        /* compare energies between sub-shortblocks */
+        attackThreshold = (chn == 3)
+            ? gfc->nsPsy.attackthre_s : gfc->nsPsy.attackthre;
+        for (i = 0; i < 12; i++)
+            if (!ns_attacks[i / 3] && attack_intensity[i] > attackThreshold)
+                ns_attacks[i / 3] = (i % 3) + 1;
 
-	if (ns_attacks[0] && gfc->nsPsy.last_attacks[chn])
-	    ns_attacks[0] = 0;
+        /* should have energy change between short blocks,
+           in order to avoid periodic signals */
+        for (i = 1; i < 4; i++) {
+            float   ratio;
+            if (en_short[i - 1] > en_short[i])
+                ratio = en_short[i - 1] / en_short[i];
+            else
+                ratio = en_short[i] / en_short[i - 1];
+            if (ratio < 1.7) {
+                ns_attacks[i] = 0;
+                if (i == 1)
+                    ns_attacks[0] = 0;
+            }
+        }
 
-	if (gfc->nsPsy.last_attacks[chn] == 3 ||
-	    ns_attacks[0] + ns_attacks[1] + ns_attacks[2] + ns_attacks[3]) {
-	    ns_uselongblock = 0;
+        if (ns_attacks[0] && gfc->nsPsy.last_attacks[chn])
+            ns_attacks[0] = 0;
+
+        if (gfc->nsPsy.last_attacks[chn] == 3 ||
+            ns_attacks[0] + ns_attacks[1] + ns_attacks[2] + ns_attacks[3]) {
+            ns_uselongblock = 0;
+
+            if (ns_attacks[1] && ns_attacks[0])
+                ns_attacks[1] = 0;
+            if (ns_attacks[2] && ns_attacks[1])
+                ns_attacks[2] = 0;
+            if (ns_attacks[3] && ns_attacks[2])
+                ns_attacks[3] = 0;
+        }
 
-	    if (ns_attacks[1] && ns_attacks[0]) ns_attacks[1] = 0;
-	    if (ns_attacks[2] && ns_attacks[1]) ns_attacks[2] = 0;
-	    if (ns_attacks[3] && ns_attacks[2]) ns_attacks[3] = 0;
-	}
     if (chn < 2) {
-    	uselongblock[chn] = ns_uselongblock;
+        uselongblock[chn] = ns_uselongblock;
     }
-    else if (ns_uselongblock == 0) {
-        uselongblock[0] = uselongblock[1] = 0;
+    else {
+        if (ns_uselongblock == 0) {
+            uselongblock[0] = uselongblock[1] = 0;
+        }
     }
 
 	/* there is a one granule delay.  Copy maskings computed last call
@@ -1495,7 +1536,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 
 	/* compute masking thresholds for short blocks */
 	for (sblock = 0; sblock < 3; sblock++) {
-	    FLOAT8 enn, thmm;
+	    FLOAT enn, thmm;
 	    compute_masking_s(gfc, fftenergy_s, eb, thr, chn, sblock,
 			      gfp->ATHlower*gfc->ATH->adjust);
         b = -1;
@@ -1555,7 +1596,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	    FLOAT ebb,m;
 	    m = ebb = fftenergy[j++];
 	    for (i = gfc->numlines_l[b] - 1; i > 0; --i) {
-		FLOAT8 el = fftenergy[j++];
+		FLOAT el = fftenergy[j++];
 		ebb += el;
 		if (m < el)
 		    m = el;
@@ -1568,7 +1609,7 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	if (gfc->nsPsy.pass1fp)
 	    nsPsy2dataRead(gfc->nsPsy.pass1fp, eb2, eb, chn, gfc->npart_l);
 	else {
-	    FLOAT8 m,a;
+	    FLOAT m,a;
 	    a = avg[0] + avg[1];
 	    if (a != 0.0) {
 		m = max[0]; if (m < max[1]) m = max[1];
@@ -1617,9 +1658,14 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 	 *      convolve the partitioned energy and unpredictability
 	 *      with the spreading function, s3_l[b][k]
 	 ******************************************************************* */
+#undef GPSYCHO_BLOCK_TYPE_DECISION
+#ifdef GPSYCHO_BLOCK_TYPE_DECISION
+   {
+    FLOAT pe = 0;
+#endif
 	k = 0;
 	for ( b = 0;b < gfc->npart_l; b++ ) {
-	    FLOAT8 ecb;
+	    FLOAT ecb;
 	    /* convolve the partitioned energy with the spreading function */
 	    int kk = gfc->s3ind[b][0];
 	    ecb = gfc->s3_ll[k++] * eb2[kk];
@@ -1650,8 +1696,18 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 
 	    gfc->nb_2[chn][b] = gfc->nb_1[chn][b];
 	    gfc->nb_1[chn][b] = ecb;
-	}
-
+#ifdef GPSYCHO_BLOCK_TYPE_DECISION
+        /* this pe does not match GPSYCHO's pe, because of difference in 
+         * convolution calculation, (mask_add etc.). Therefore the block
+         * switching does not happen exactly as in GPSYCHO.
+         */
+		pe -= gfc->numlines_l[b] * FAST_LOG(ecb / eb[b]);
+#endif
+    }
+#ifdef GPSYCHO_BLOCK_TYPE_DECISION
+    determine_block_type( gfp, fftenergy_s, uselongblock, chn, gr_out, &pe );
+   }
+#endif
 	/* compute masking thresholds for long blocks */
 	convert_partition2scalefac_l(gfc, eb, thr, chn);
 
@@ -1712,8 +1768,8 @@ int L3psycho_anal_ns( lame_global_flags * gfp,
 /* 
  *   The spreading function.  Values returned in units of energy
  */
-static FLOAT8 s3_func(FLOAT8 bark) {
-    FLOAT8 tempx,x,tempy,temp;
+static FLOAT s3_func(FLOAT bark) {
+    FLOAT tempx,x,tempy,temp;
     tempx = bark;
     if (tempx>=0) tempx *= 3;
     else tempx *=1.5; 
@@ -1745,10 +1801,10 @@ static FLOAT8 s3_func(FLOAT8 bark) {
 static int
 init_numline(
     int *numlines, int *bo, int *bm,
-    FLOAT8 *bval, FLOAT8 *bval_width, FLOAT8 *mld,
+    FLOAT *bval, FLOAT *bval_width, FLOAT *mld,
 
-    FLOAT8 sfreq, int blksize, int *scalepos,
-    FLOAT8 deltafreq, int sbmax
+    FLOAT sfreq, int blksize, int *scalepos,
+    FLOAT deltafreq, int sbmax
     )
 {
     int partition[HBLKSIZE];
@@ -1760,7 +1816,7 @@ init_numline(
     /* compute numlines, the number of spectral lines in each partition band */
     /* each partition band should be about DELBARK wide. */
     for (i=0;i<CBANDS;i++) {
-	FLOAT8 bark1;
+	FLOAT bark1;
 	int j2;
 	bark1 = freq2bark(sfreq*j);
 	for (j2 = j; freq2bark(sfreq*j2) - bark1 < DELBARK && j2 <= blksize/2;
@@ -1775,7 +1831,7 @@ init_numline(
 
     for ( sfb = 0; sfb < sbmax; sfb++ ) {
 	int i1,i2,start,end;
-	FLOAT8 arg;
+	FLOAT arg;
 	start = scalepos[sfb];
 	end   = scalepos[sfb+1];
 
@@ -1799,7 +1855,7 @@ init_numline(
     j = 0;
     for (k = 0; k < i+1; k++) {
 	int w = numlines[k];
-	FLOAT8  bark1,bark2;
+	FLOAT  bark1,bark2;
 
 	bark1 = freq2bark (sfreq*(j    ));
 	bark2 = freq2bark (sfreq*(j+w-1));
@@ -1820,9 +1876,9 @@ init_s3_values(
     FLOAT **p,
     int (*s3ind)[2],
     int npart,
-    FLOAT8 *bval,
-    FLOAT8 *bval_width,
-    FLOAT8 *norm
+    FLOAT *bval,
+    FLOAT *bval_width,
+    FLOAT *norm
     )
 {
     FLOAT s3[CBANDS][CBANDS];
@@ -1873,10 +1929,10 @@ int psymodel_init(lame_global_flags *gfp)
     lame_internal_flags *gfc=gfp->internal_flags;
     int i,j,b,sb,k;
 
-    FLOAT8 bval[CBANDS];
-    FLOAT8 bval_width[CBANDS];
-    FLOAT8 norm[CBANDS];
-    FLOAT8 sfreq = gfp->out_samplerate;
+    FLOAT bval[CBANDS];
+    FLOAT bval_width[CBANDS];
+    FLOAT norm[CBANDS];
+    FLOAT sfreq = gfp->out_samplerate;
 
     gfc->ms_ener_ratio_old=.25;
     gfc->blocktype_old[0] = gfc->blocktype_old[1] = NORM_TYPE; /* the vbr header is long blocks*/
@@ -1948,8 +2004,8 @@ int psymodel_init(lame_global_flags *gfp)
 	/* ATH */
 	x = FLOAT_MAX;
 	for (k=0; k < gfc->numlines_l[i]; k++, j++) {
-	    FLOAT8  freq = sfreq*j/(1000.0*BLKSIZE);
-	    FLOAT8  level;
+	    FLOAT  freq = sfreq*j/(1000.0*BLKSIZE);
+	    FLOAT  level;
 	    /*	freq = Min(.1,freq);*/      /* ATH below 100 Hz constant, not further climbing */
 	    level  = ATHformula (freq*1000, gfp) - 20;   /* scale to FFT units; returned value is in dB */
 	    level  = pow ( 10., 0.1*level );   /* convert from dB -> energy */
