@@ -1,7 +1,10 @@
 /*
  * MP3 quantization
  *
- * Copyright (c) 1999 Mark Taylor
+ *	Copyright (c) 1999-2000 Mark Taylor
+ *	Copyright (c) 1999-2003 Takehiro Tominaga
+ *	Copyright (c) 2000-2005 Robert Hegemann
+ *	Copyright (c) 2001-2005 Gabriel Bouvigne
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,7 +22,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: quantize.c,v 1.151 2004/04/03 17:28:32 bouvigne Exp $ */
+/* $Id: quantize.c,v 1.170.2.2 2005/11/20 14:08:25 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -36,6 +39,7 @@
 #include "lame-analysis.h"
 #include "vbrquantize.h"
 #include "machine.h"
+#include "util.h"
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -50,7 +54,7 @@ ms_sparsing(lame_internal_flags* gfc, int gr)
 {
     int sfb, m, k, i, j = 0;
     int width;
-    FLOAT8 treshold = 0;
+    FLOAT treshold = 0;
     
     if ( gfc->sparsing == 0 ) return;
     m = gfc->l3_side.tt[gr][0].sfb_lmax;
@@ -58,18 +62,18 @@ ms_sparsing(lame_internal_flags* gfc, int gr)
         width = gfc->scalefac_band.l[sfb+1] - gfc->scalefac_band.l[sfb];
         treshold = pow( 10, -(gfc->sparseA-gfc->mld_l[sfb]*gfc->sparseB)/10.);
         for ( i = 0; i < width; i += 2, j += 2 ) {
-            FLOAT8* m0 = gfc->l3_side.tt[gr][0].xr+j;
-            FLOAT8* m1 = gfc->l3_side.tt[gr][0].xr+j+1;
-            FLOAT8* s0 = gfc->l3_side.tt[gr][1].xr+j;
-            FLOAT8* s1 = gfc->l3_side.tt[gr][1].xr+j+1;
-            FLOAT8 m02 = *m0 * *m0;
-            FLOAT8 m12 = *m1 * *m1;
-            FLOAT8 s02 = *s0 * *s0;
-            FLOAT8 s12 = *s1 * *s1;
-            FLOAT8 u0 = m02*treshold;
-            FLOAT8 u1 = m12*treshold;
-            FLOAT8 v0 = s02*treshold;
-            FLOAT8 v1 = s12*treshold;
+            FLOAT* m0 = gfc->l3_side.tt[gr][0].xr+j;
+            FLOAT* m1 = gfc->l3_side.tt[gr][0].xr+j+1;
+            FLOAT* s0 = gfc->l3_side.tt[gr][1].xr+j;
+            FLOAT* s1 = gfc->l3_side.tt[gr][1].xr+j+1;
+            FLOAT m02 = *m0 * *m0;
+            FLOAT m12 = *m1 * *m1;
+            FLOAT s02 = *s0 * *s0;
+            FLOAT s12 = *s1 * *s1;
+            FLOAT u0 = m02*treshold;
+            FLOAT u1 = m12*treshold;
+            FLOAT v0 = s02*treshold;
+            FLOAT v1 = s12*treshold;
             if ( s02 < u0 && s12 < u1 ) { 
                 *m0 += *s0; *s0 = 0; 
                 *m1 += *s1; *s1 = 0; 
@@ -85,18 +89,18 @@ ms_sparsing(lame_internal_flags* gfc, int gr)
         treshold = pow( 10, -(gfc->sparseA-gfc->mld_s[sfb]*gfc->sparseB)/10.);
         for ( i = 0; i < width; i += 2, j += 6 ) 
         for ( k = 0; k < 3; ++k ) {
-            FLOAT8* m0 = gfc->l3_side.tt[gr][0].xr+j+k;
-            FLOAT8* m1 = gfc->l3_side.tt[gr][0].xr+j+k+3;
-            FLOAT8* s0 = gfc->l3_side.tt[gr][1].xr+j+k;
-            FLOAT8* s1 = gfc->l3_side.tt[gr][1].xr+j+k+3;
-            FLOAT8 m02 = *m0 * *m0;
-            FLOAT8 m12 = *m1 * *m1;
-            FLOAT8 s02 = *s0 * *s0;
-            FLOAT8 s12 = *s1 * *s1;
-            FLOAT8 u0 = m02*treshold;
-            FLOAT8 u1 = m12*treshold;
-            FLOAT8 v0 = s02*treshold;
-            FLOAT8 v1 = s12*treshold;
+            FLOAT* m0 = gfc->l3_side.tt[gr][0].xr+j+k;
+            FLOAT* m1 = gfc->l3_side.tt[gr][0].xr+j+k+3;
+            FLOAT* s0 = gfc->l3_side.tt[gr][1].xr+j+k;
+            FLOAT* s1 = gfc->l3_side.tt[gr][1].xr+j+k+3;
+            FLOAT m02 = *m0 * *m0;
+            FLOAT m12 = *m1 * *m1;
+            FLOAT s02 = *s0 * *s0;
+            FLOAT s12 = *s1 * *s1;
+            FLOAT u0 = m02*treshold;
+            FLOAT u1 = m12*treshold;
+            FLOAT v0 = s02*treshold;
+            FLOAT v1 = s12*treshold;
             if ( s02 < u0 && s12 < u1 ) { 
                 *m0 += *s0; *s0 = 0; 
                 *m1 += *s1; *s1 = 0; 
@@ -111,10 +115,10 @@ ms_sparsing(lame_internal_flags* gfc, int gr)
     if ( gfc->sparsing == 2 ) {
         for ( ; j < 575; j += 2*m ) 
         for ( i = 0; i < m; ++i ) {
-            FLOAT8* m0 = gfc->l3_side.tt[gr][0].xr+j+i;
-            FLOAT8* m1 = gfc->l3_side.tt[gr][0].xr+j+i+m;
-            FLOAT8* s0 = gfc->l3_side.tt[gr][1].xr+j+i;
-            FLOAT8* s1 = gfc->l3_side.tt[gr][1].xr+j+i+m;
+            FLOAT* m0 = gfc->l3_side.tt[gr][0].xr+j+i;
+            FLOAT* m1 = gfc->l3_side.tt[gr][0].xr+j+i+m;
+            FLOAT* s0 = gfc->l3_side.tt[gr][1].xr+j+i;
+            FLOAT* s1 = gfc->l3_side.tt[gr][1].xr+j+i+m;
             *m0 += *s0; *s0 = 0; 
             *m1 += *s1; *s1 = 0; 
         }
@@ -122,18 +126,18 @@ ms_sparsing(lame_internal_flags* gfc, int gr)
     else {
     for ( ; j < 575; j += 2*m ) 
     for ( i = 0; i < m; ++i ) {
-        FLOAT8* m0 = gfc->l3_side.tt[gr][0].xr+j+i;
-        FLOAT8* m1 = gfc->l3_side.tt[gr][0].xr+j+i+m;
-        FLOAT8* s0 = gfc->l3_side.tt[gr][1].xr+j+i;
-        FLOAT8* s1 = gfc->l3_side.tt[gr][1].xr+j+i+m;
-        FLOAT8 m02 = *m0 * *m0;
-        FLOAT8 m12 = *m1 * *m1;
-        FLOAT8 s02 = *s0 * *s0;
-        FLOAT8 s12 = *s1 * *s1;
-        FLOAT8 u0 = m02*treshold;
-        FLOAT8 u1 = m12*treshold;
-        FLOAT8 v0 = s02*treshold;
-        FLOAT8 v1 = s12*treshold;
+        FLOAT* m0 = gfc->l3_side.tt[gr][0].xr+j+i;
+        FLOAT* m1 = gfc->l3_side.tt[gr][0].xr+j+i+m;
+        FLOAT* s0 = gfc->l3_side.tt[gr][1].xr+j+i;
+        FLOAT* s1 = gfc->l3_side.tt[gr][1].xr+j+i+m;
+        FLOAT m02 = *m0 * *m0;
+        FLOAT m12 = *m1 * *m1;
+        FLOAT s02 = *s0 * *s0;
+        FLOAT s12 = *s1 * *s1;
+        FLOAT u0 = m02*treshold;
+        FLOAT u1 = m12*treshold;
+        FLOAT v0 = s02*treshold;
+        FLOAT v1 = s12*treshold;
         if ( s02 < u0 && s12 < u1 ) { 
             *m0 += *s0; *s0 = 0; 
             *m1 += *s1; *s1 = 0; 
@@ -153,11 +157,11 @@ ms_convert(III_side_info_t *l3_side, int gr)
 {
     int i;
     for (i = 0; i < 576; ++i) {
-	FLOAT8 l, r;
+	FLOAT l, r;
         l = l3_side->tt[gr][0].xr[i];
         r = l3_side->tt[gr][1].xr[i];
-        l3_side->tt[gr][0].xr[i] = (l+r) * (FLOAT8)(SQRT2*0.5);
-        l3_side->tt[gr][1].xr[i] = (l-r) * (FLOAT8)(SQRT2*0.5);
+        l3_side->tt[gr][0].xr[i] = (l+r) * (FLOAT)(SQRT2*0.5);
+        l3_side->tt[gr][1].xr[i] = (l-r) * (FLOAT)(SQRT2*0.5);
     }
 }
 
@@ -172,14 +176,118 @@ ms_convert(III_side_info_t *l3_side, int gr)
  *
  ************************************************************************/
 
+
+void init_xrpow_core_c(gr_info *const cod_info, 
+                        FLOAT xrpow[576],
+                        int upper,
+                        FLOAT* sum)
+{
+    int i;
+    FLOAT tmp;
+    *sum = 0;
+    for (i = 0; i <= upper; ++i) {
+        tmp = fabs (cod_info->xr[i]);
+        *sum += tmp;
+        xrpow[i] = sqrt (tmp * sqrt(tmp));
+
+        if (xrpow[i] > cod_info->xrpow_max)
+            cod_info->xrpow_max = xrpow[i];
+    }
+}
+
+
+#ifdef HAVE_XMMINTRIN_H
+
+#include <xmmintrin.h>
+
+typedef union {
+    __m128   _m128;
+    float     _float[4];
+} vecfloat_union;
+
+
+
+void init_xrpow_core_sse(gr_info *const cod_info, 
+                        FLOAT xrpow[576],
+                        int upper,
+                        FLOAT* sum)
+{
+    int i;
+    FLOAT tmp;
+    int upper4 = (upper/4)*4;
+
+
+    const int fabs_mask[4] = {0x7FFFFFFF, 0x7FFFFFFF, 0x7FFFFFFF, 0x7FFFFFFF};
+    const __m128 vec_fabs_mask = _mm_loadu_ps((float *) fabs_mask);
+    vecfloat_union vec_xrpow_max;
+    vecfloat_union vec_sum;
+
+    _mm_prefetch((char*)cod_info->xr, _MM_HINT_T0);
+    _mm_prefetch((char*)xrpow, _MM_HINT_T0);
+
+    vec_xrpow_max._m128 = _mm_set_ps1(0);
+    vec_sum._m128 = _mm_set_ps1(0);
+
+    for (i = 0; i < upper4; i+=4) {
+        __m128 vec_tmp;
+
+        vec_tmp = _mm_loadu_ps(&(cod_info->xr[i])); //load
+        vec_tmp = _mm_and_ps(vec_tmp, vec_fabs_mask); //fabs
+
+        vec_sum._m128 = _mm_add_ps(vec_sum._m128, vec_tmp);
+
+        vec_tmp = _mm_sqrt_ps(_mm_mul_ps(vec_tmp, _mm_sqrt_ps(vec_tmp)));
+
+        _mm_storeu_ps(&(xrpow[i]), vec_tmp); //store into xrpow[]
+
+        vec_xrpow_max._m128 = _mm_max_ps(vec_xrpow_max._m128, vec_tmp); //retrieve max
+    }
+
+    *sum = vec_sum._float[0] + vec_sum._float[1] + vec_sum._float[2] + vec_sum._float[3];
+    cod_info->xrpow_max = Max(vec_xrpow_max._float[0],
+                                Max(vec_xrpow_max._float[1],
+                                Max(vec_xrpow_max._float[2], vec_xrpow_max._float[3])));
+
+
+
+    for (i = upper4; i <= upper; ++i) {
+        tmp = fabs (cod_info->xr[i]);
+        *sum += tmp;
+        xrpow[i] = sqrt (tmp * sqrt(tmp));
+
+        if (xrpow[i] > cod_info->xrpow_max)
+            cod_info->xrpow_max = xrpow[i];
+    }
+}
+
+
+#endif
+
+
+
+
+
+void init_xrpow_core_init(lame_internal_flags * const gfc)
+{
+    gfc->init_xrpow_core = init_xrpow_core_c;
+    
+#ifdef HAVE_XMMINTRIN_H
+    if (gfc->CPU_features.SSE)
+        gfc->init_xrpow_core = init_xrpow_core_sse;
+#endif
+}
+
+
+
 static int 
 init_xrpow(
     lame_internal_flags *gfc,
     gr_info *const cod_info, 
-    FLOAT8 xrpow[576] )
+    FLOAT xrpow[576] )
 {
-    FLOAT8 tmp, sum = 0;
+    FLOAT sum = 0;
     int i;
+    int upper = cod_info->max_nonzero_coeff;
 
     assert( xrpow != NULL );
     cod_info->xrpow_max = 0;
@@ -187,17 +295,14 @@ init_xrpow(
     /*  check if there is some energy we have to quantize
      *  and calculate xrpow matching our fresh scalefactors
      */
-    for (i = 0; i < 576; ++i) {
-        tmp = fabs (cod_info->xr[i]);
-        sum += tmp;
-        xrpow[i] = sqrt (tmp * sqrt(tmp));
+    memset(&(xrpow[upper]), 0, (575-upper)*sizeof(xrpow[upper]));
+
+
+    gfc->init_xrpow_core(cod_info, xrpow, upper, &sum);
 
-        if (xrpow[i] > cod_info->xrpow_max)
-            cod_info->xrpow_max = xrpow[i];
-    }
     /*  return 1 if we have something to quantize, else 0
      */
-    if (sum > (FLOAT8)1E-20) {
+    if (sum > (FLOAT)1E-20) {
         int j = 0;
         if (gfc->substep_shaping & 2)
             j = 1;
@@ -228,10 +333,6 @@ or sfb12 for short blocks
 From top to bottom of sfb, changes to 0
 coeffs which are below ath. It stops on the first
 coeff higher than ath.
-
-It should help reducing bitrate,
-but real gain is only about 3kbps at max
-and sometimes it even increases bitrate.
 */
 void psfb21_analogsilence(
         lame_global_flags *gfp,
@@ -240,7 +341,7 @@ void psfb21_analogsilence(
     )
 {
     ATH_t * ATH = gfc->ATH;
-    FLOAT8 *xr = cod_info->xr;
+    FLOAT *xr = cod_info->xr;
 
     if (cod_info->block_type == NORM_TYPE) {
         int gsfb;
@@ -249,11 +350,11 @@ void psfb21_analogsilence(
             int start = gfc->scalefac_band.psfb21[ gsfb ];
             int end = gfc->scalefac_band.psfb21[ gsfb+1 ];
             int j;
-            FLOAT8 ath21;
-            if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)
-                ath21 = athAdjust(ATH->adjust, ATH->psfb21[gsfb], ATH->floor);
-            else
-                ath21 = ATH->adjust * ATH->psfb21[gsfb];
+            FLOAT ath21;
+            ath21 = athAdjust(ATH->adjust, ATH->psfb21[gsfb], ATH->floor);
+
+            if (gfc->nsPsy.longfact[21] != 0)
+                ath21 *= gfc->nsPsy.longfact[21];
 
             for (j = end-1; j>=start; j--) {
                 if ( fabs(xr[j]) < ath21)
@@ -277,11 +378,11 @@ void psfb21_analogsilence(
                             (gfc->scalefac_band.psfb12[gsfb] - gfc->scalefac_band.psfb12[0]);
                 int end = start + (gfc->scalefac_band.psfb12[gsfb+1] - gfc->scalefac_band.psfb12[gsfb]);
                 int j;
-                FLOAT8 ath12;
-                if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)
-                    ath12 = athAdjust(ATH->adjust, ATH->psfb12[gsfb], ATH->floor);
-                else
-                    ath12 = ATH->adjust * ATH->psfb12[gsfb];
+                FLOAT ath12;
+                ath12 = athAdjust(ATH->adjust, ATH->psfb12[gsfb], ATH->floor);
+
+                if (gfc->nsPsy.shortfact[12] != 0)
+                    ath12 *= gfc->nsPsy.shortfact[12];
 
                 for (j = end-1; j>=start; j--) {
                     if ( fabs(xr[j]) < ath12)
@@ -341,8 +442,8 @@ init_outer_loop(
 	cod_info->window[sfb] = 3; /* which is always 0. */
     }
     if (cod_info->block_type == SHORT_TYPE) {
-	FLOAT8 ixwork[576];
-	FLOAT8 *ix;
+	FLOAT ixwork[576];
+	FLOAT *ix;
 
         cod_info->sfb_smin        = 0;
         cod_info->sfb_lmax        = 0;
@@ -370,7 +471,7 @@ init_outer_loop(
 	  order of increasing frequency...
 	*/
 	ix = &cod_info->xr[gfc->scalefac_band.l[cod_info->sfb_lmax]];
-	memcpy(ixwork, cod_info->xr, 576*sizeof(FLOAT8));
+	memcpy(ixwork, cod_info->xr, 576*sizeof(FLOAT));
 	for (sfb = cod_info->sfb_smin; sfb < SBMAX_s; sfb++) {
 	    int start = gfc->scalefac_band.s[sfb];
 	    int end   = gfc->scalefac_band.s[sfb + 1];
@@ -434,7 +535,7 @@ bin_search_StepSize(
           gr_info * const cod_info,
 	 int             desired_rate,
     const int             ch,
-    const FLOAT8          xrpow [576] ) 
+    const FLOAT          xrpow [576] ) 
 {
     int nBits;
     int CurrentStep = gfc->CurrentStep[ch];
@@ -501,9 +602,9 @@ bin_search_StepSize(
  *
  ************************************************************************/
 static int
-float8compare(const void * v1, const void * v2)
+floatcompare(const void * v1, const void * v2)
 {
-    const FLOAT8 *a = v1, *b = v2;
+    const FLOAT *a = v1, *b = v2;
     if (*a > *b) return 1;
     if (*a == *b) return 0;
     return -1;
@@ -513,12 +614,12 @@ void
 trancate_smallspectrums(
     lame_internal_flags *gfc,
     gr_info		* const gi,
-    const FLOAT8	* const l3_xmin,
-    FLOAT8		* work
+    const FLOAT	* const l3_xmin,
+    FLOAT		* work
     )
 {
     int sfb, j, width;
-    FLOAT8 distort[SFBMAX];
+    FLOAT distort[SFBMAX];
     calc_noise_result dummy;
 
     if ((!(gfc->substep_shaping & 4) && gi->block_type == SHORT_TYPE)
@@ -526,7 +627,7 @@ trancate_smallspectrums(
 	return;
     calc_noise (gfc, gi, l3_xmin, distort, &dummy, 0);
     for (j = 0; j < 576; j++) {
-	FLOAT8 xr = 0.0;
+	FLOAT xr = 0.0;
 	if (gi->l3_enc[j] != 0)
 	    xr = fabs(gi->xr[j]);
 	work[j] = xr;
@@ -537,7 +638,7 @@ trancate_smallspectrums(
     if (gi->block_type == SHORT_TYPE)
 	sfb = 6;
     do {
-	FLOAT8 allowedNoise, trancateThreshold;
+	FLOAT allowedNoise, trancateThreshold;
 	int nsame, start;
 
 	width = gi->width[sfb];
@@ -545,7 +646,7 @@ trancate_smallspectrums(
 	if (distort[sfb] >= 1.0)
 	    continue;
 
-	qsort(&work[j-width], width, sizeof(FLOAT8), float8compare);
+	qsort(&work[j-width], width, sizeof(FLOAT), floatcompare);
 	if (work[j - 1] == 0.0)
 	    continue; /* all zero sfb */
 
@@ -553,7 +654,7 @@ trancate_smallspectrums(
 	trancateThreshold = 0.0;
 	start = 0;
 	do {
-	    FLOAT8 noise;
+	    FLOAT noise;
 	    for (nsame = 1; start + nsame < width; nsame++)
 		if (work[start + j-width] != work[start+j+nsame-width])
 		    break;
@@ -584,7 +685,7 @@ trancate_smallspectrums(
 	} while (--width > 0);
     } while (++sfb < gi->psymax);
 
-    gi->part2_3_length = noquant_count_bits(gfc, gi);
+    gi->part2_3_length = noquant_count_bits(gfc, gi, 0);
 }
 
 
@@ -635,7 +736,7 @@ static double penalties ( double noise )
 }
 
 static double get_klemm_noise(
-    const FLOAT8  * distort,
+    const FLOAT  * distort,
     const gr_info * const gi
     )
 {
@@ -655,7 +756,7 @@ quant_compare(
     const calc_noise_result	* const best,
           calc_noise_result	* const calc,
     const gr_info		* const gi,
-    const FLOAT8		* distort
+    const FLOAT		* distort
     )
 {
     /*
@@ -671,22 +772,15 @@ quant_compare(
     switch (quant_comp) {
         default:
         case 9: {
-            /* minimize var_noise in order to shape noise */
             if (best->over_count > 0 ) {
                 /* there are distorted sfb*/
-	            better = (calc->over_count == 0)
-                        || ( calc->over_noise <= best->over_noise &&
-                             calc->var_noise  <= best->var_noise )
-                        || ( calc->over_noise <= best->over_noise &&
-                             calc->max_noise  <= best->max_noise );
+	            better = calc->over_SSD <= best->over_SSD;
+                if (calc->over_SSD == best->over_SSD)
+                    better = calc->bits < best->bits;
             } else {
                 /* no distorted sfb*/
-                if (calc->over_count > 0) {
-                    better = 0;
-                } else {
-		            better = (calc->tot_noise < best->tot_noise)
-		                &&   (calc->var_noise <= best->var_noise);
-                }
+                better = ((calc->max_noise < 0) &&
+                          ((calc->max_noise*10 + calc->bits) <= (best->max_noise*10 + best->bits)));
             }
 	        break;
         }
@@ -752,6 +846,19 @@ quant_compare(
 	    break;
     }   
 
+
+    if (best->over_count == 0) {
+        /*
+            If no distorted bands, only use this quantization
+            if it is better, and if it uses less bits.
+            Unfortunately, part2_3_length is sometimes a poor
+            estimator of the final size at low bitrates.
+        */
+        better = better &&
+                calc->bits < best->bits;
+    }
+
+
     return better;
 }
 
@@ -791,12 +898,14 @@ static void
 amp_scalefac_bands(
     lame_global_flags *gfp,
     gr_info  *const cod_info, 
-    FLOAT8 *distort,
-    FLOAT8 xrpow[576] )
+    FLOAT *distort,
+    FLOAT xrpow[576],
+    int bRefine)
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     int j, sfb;
-    FLOAT8 ifqstep34, trigger;
+    FLOAT ifqstep34, trigger;
+    int noise_shaping_amp;
 
     if (cod_info->scalefac_scale == 0) {
 	ifqstep34 = 1.29683955465100964055; /* 2**(.75*.5)*/
@@ -811,7 +920,14 @@ amp_scalefac_bands(
 	    trigger = distort[sfb];
     }
 
-    switch (gfc->noise_shaping_amp) {
+    noise_shaping_amp = gfc->noise_shaping_amp;
+    if (noise_shaping_amp == 3) {
+        if (bRefine == 1)
+            noise_shaping_amp = 2;
+        else
+            noise_shaping_amp = 1;
+    }
+    switch (noise_shaping_amp) {
     case 2:
 	/* amplify exactly 1 band */
 	break;
@@ -872,10 +988,10 @@ amp_scalefac_bands(
 static void
 inc_scalefac_scale (
     gr_info        * const cod_info, 
-    FLOAT8                 xrpow[576] )
+    FLOAT                 xrpow[576] )
 {
     int l, j, sfb;
-    const FLOAT8 ifqstep34 = 1.29683955465100964055;
+    const FLOAT ifqstep34 = 1.29683955465100964055;
 
     j = 0;
     for (sfb = 0; sfb < cod_info->sfbmax; sfb++) {
@@ -914,7 +1030,7 @@ static int
 inc_subblock_gain (
     const lame_internal_flags        * const gfc,
           gr_info        * const cod_info,
-          FLOAT8                 xrpow[576] )
+          FLOAT                 xrpow[576] )
 {
     int sfb, window;
     int * const scalefac = cod_info->scalefac;
@@ -953,7 +1069,7 @@ inc_subblock_gain (
         j = gfc->scalefac_band.l[cod_info->sfb_lmax];
         for (sfb = cod_info->sfb_lmax+window;
         sfb < cod_info->sfbmax; sfb += 3) {
-            FLOAT8 amp;
+            FLOAT amp;
             int width = cod_info->width[sfb], s = scalefac[sfb];
             assert(s >= 0);
             s = s - (4 >> cod_info->scalefac_scale);
@@ -975,7 +1091,7 @@ inc_subblock_gain (
         }
 
         {
-            FLOAT8 amp = IPOW20(210 - 8);
+            FLOAT amp = IPOW20(210 - 8);
             j += cod_info->width[sfb] * (window+1);
             for (l = -cod_info->width[sfb]; l < 0; l++) {
                 xrpow[j+l] *= amp;
@@ -1009,13 +1125,14 @@ static int
 balance_noise (
     lame_global_flags  *const gfp,
     gr_info        * const cod_info,
-    FLOAT8         * distort,
-    FLOAT8         xrpow[576] )
+    FLOAT         * distort,
+    FLOAT         xrpow[576],
+    int bRefine)
 {
     lame_internal_flags *const gfc = gfp->internal_flags;
     int status;
     
-    amp_scalefac_bands ( gfp, cod_info, distort, xrpow);
+    amp_scalefac_bands ( gfp, cod_info, distort, xrpow, bRefine);
 
     /* check to make sure we have not amplified too much 
      * loop_break returns 0 if there is an unamplified scalefac
@@ -1087,21 +1204,25 @@ static int
 outer_loop (
     lame_global_flags	*gfp,
     gr_info		* const cod_info,
-    const FLOAT8	* const l3_xmin,  /* allowed distortion */
-    FLOAT8		xrpow[576], /* coloured magnitudes of spectral */
+    const FLOAT	* const l3_xmin,  /* allowed distortion */
+    FLOAT		xrpow[576], /* coloured magnitudes of spectral */
     const int           ch,
     const int           targ_bits )  /* maximum allowed bits */
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     gr_info cod_info_w;
-    FLOAT8 save_xrpow[576];
-    FLOAT8 distort[SFBMAX];
+    FLOAT save_xrpow[576];
+    FLOAT distort[SFBMAX];
     calc_noise_result best_noise_info;
     int huff_bits;
     int better;
     int over;
     int age;
     calc_noise_data prev_noise;
+    int best_part2_3_length = 9999999;
+    int bEndOfSearch = 0;
+    int bRefine = 0;
+    int best_ggain_pass1 = 0;
 
     bin_search_StepSize (gfc, cod_info, targ_bits, ch, xrpow);
 
@@ -1115,110 +1236,154 @@ outer_loop (
     /* compute the distortion in this quantization */
     /* coefficients and thresholds both l/r (or both mid/side) */
     over = calc_noise (gfc, cod_info, l3_xmin, distort, &best_noise_info, &prev_noise);
+    best_noise_info.bits = cod_info->part2_3_length;
+
     cod_info_w = *cod_info;
     age = 0;
-    if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)
-	memcpy(save_xrpow, xrpow, sizeof(FLOAT8)*576);
+   /* if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)*/
+	memcpy(save_xrpow, xrpow, sizeof(FLOAT)*576);
 
-    /* BEGIN MAIN LOOP */
-    do {
-	calc_noise_result noise_info;
-	/******************************************************************/
-	/* stopping criterion */
-	/******************************************************************/
-	/* if no bands with distortion and -X0, we are done */
-	if (0==gfc->noise_shaping_stop && 
-	    0==gfp->quant_comp &&
-	    (over == 0 || best_noise_info.over_count == 0) )
-	    break;
+    while (!bEndOfSearch) {
+        /* BEGIN MAIN LOOP */
+        do {
+	        calc_noise_result noise_info;
+            int search_limit;
+            int maxggain = 255;
 
-	/* Check if the last scalefactor band is distorted.
-	 * in VBR mode we can't get rid of the distortion, so quit now
-	 * and VBR mode will try again with more bits.  
-	 * (makes a 10% speed increase, the files I tested were
-	 * binary identical, 2000/05/20 Robert Hegemann)
-	 * distort[] > 1 means noise > allowed noise
-	 */
-	if (gfc->sfb21_extra) {
-	    if (distort[cod_info_w.sfbmax] > 1.0)
-		break;
-	    if (cod_info_w.block_type == SHORT_TYPE
-		&& (distort[cod_info_w.sfbmax+1] > 1.0
-		    || distort[cod_info_w.sfbmax+2] > 1.0))
-		    break;
-	}
+            /* When quantization with no distorted bands is found,
+             * allow up to X new unsuccesful tries in serial. This
+             * gives us more possibilities for different quant_compare modes.
+             * Much more than 3 makes not a big difference, it is only slower.
+             */
 
-	/* try the new scalefactor conbination on cod_info_w */
-	if (balance_noise (gfp, &cod_info_w, distort, xrpow) == 0)
-	    break;
-
-        /* inner_loop starts with the initial quantization step computed above
-         * and slowly increases until the bits < huff_bits.
-         * Thus it is important not to start with too large of an inital
-         * quantization step.  Too small is ok, but inner_loop will take longer
-         */
-        huff_bits = targ_bits - cod_info_w.part2_length;
-        if (huff_bits <= 0)
-            break;
-
-	/*  increase quantizer stepsize until needed bits are below maximum
-	 */
-	while ((cod_info_w.part2_3_length
-		= count_bits(gfc, xrpow, &cod_info_w, &prev_noise)) > huff_bits
-	       && cod_info_w.global_gain < 256u)
-	    cod_info_w.global_gain++;
-
-	if (cod_info_w.global_gain >= 256)
-	    break;
-
-        /* compute the distortion in this quantization */
-	over = calc_noise (gfc, &cod_info_w, l3_xmin, distort, &noise_info, &prev_noise);
-
-        /* check if this quantization is better
-         * than our saved quantization */
-	if (cod_info->block_type == NORM_TYPE)
-	    better = gfp->quant_comp;
-    else
-	    better = gfp->quant_comp_short;
+            if (gfc->substep_shaping & 2) {
+	            search_limit = 20;
+            }else {
+	            search_limit = 3;
+            }
 
 
-	better = quant_compare(better, gfc, &best_noise_info, &noise_info,
-			       &cod_info_w, distort);
 
-        /* save data so we can restore this quantization later */
-	if (better) {
-	    best_noise_info = noise_info;
-	    *cod_info = cod_info_w;
-	    age = 0;
-	    /* save data so we can restore this quantization later */
-	    if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh) {
-		/* store for later reuse */
-		memcpy(save_xrpow, xrpow, sizeof(FLOAT8)*576);
-	    }
-        }
-        else {
-	    /* allow up to 3 unsuccesful tries in serial, then stop 
-	     * if our best quantization so far had no distorted bands. This
-	     * gives us more possibilities for different quant_compare modes.
-	     * Much more than 3 makes not a big difference, it is only slower.
-	     */
-        if (gfc->substep_shaping & 2) {
-	        if (++age > 20 && best_noise_info.over_count == 0)
-		        break;
-        }else {
-	        if (++age > 3 && best_noise_info.over_count == 0)
+	        /* Check if the last scalefactor band is distorted.
+	         * in VBR mode we can't get rid of the distortion, so quit now
+	         * and VBR mode will try again with more bits.  
+	         * (makes a 10% speed increase, the files I tested were
+	         * binary identical, 2000/05/20 Robert Hegemann)
+	         * distort[] > 1 means noise > allowed noise
+	         */
+	        if (gfc->sfb21_extra) {
+	            if (distort[cod_info_w.sfbmax] > 1.0)
 		        break;
+	            if (cod_info_w.block_type == SHORT_TYPE
+		        && (distort[cod_info_w.sfbmax+1] > 1.0
+		            || distort[cod_info_w.sfbmax+2] > 1.0))
+		            break;
+	        }
+
+	        /* try a new scalefactor conbination on cod_info_w */
+	        if (balance_noise (gfp, &cod_info_w, distort, xrpow, bRefine) == 0)
+	            break;
+            if (cod_info_w.scalefac_scale)
+                maxggain = 254;
+
+            /* inner_loop starts with the initial quantization step computed above
+             * and slowly increases until the bits < huff_bits.
+             * Thus it is important not to start with too large of an inital
+             * quantization step.  Too small is ok, but inner_loop will take longer
+             */
+            huff_bits = targ_bits - cod_info_w.part2_length;
+            if (huff_bits <= 0)
+                break;
+
+	        /*  increase quantizer stepsize until needed bits are below maximum
+	         */
+	        while ((cod_info_w.part2_3_length
+		        = count_bits(gfc, xrpow, &cod_info_w, &prev_noise)) > huff_bits
+	               && cod_info_w.global_gain <= maxggain)
+	            cod_info_w.global_gain++;
+
+	        if (cod_info_w.global_gain > maxggain)
+	            break;
+
+            if (best_noise_info.over_count == 0) {
+
+	            while ((cod_info_w.part2_3_length
+		            = count_bits(gfc, xrpow, &cod_info_w, &prev_noise)) > best_part2_3_length
+	                   && cod_info_w.global_gain <= maxggain)
+	                cod_info_w.global_gain++;
+
+	            if (cod_info_w.global_gain > maxggain)
+	                break;
+            }
+                
+                /* compute the distortion in this quantization */
+	        over = calc_noise (gfc, &cod_info_w, l3_xmin, distort, &noise_info, &prev_noise);
+            noise_info.bits = cod_info_w.part2_3_length;
+
+                /* check if this quantization is better
+                 * than our saved quantization */
+	        if (cod_info->block_type == NORM_TYPE)
+	            better = gfp->quant_comp;
+            else
+	            better = gfp->quant_comp_short;
+
+
+	        better = quant_compare(better, gfc, &best_noise_info, &noise_info,
+			               &cod_info_w, distort);
+
+
+            /* save data so we can restore this quantization later */
+	        if (better) {
+                best_part2_3_length = cod_info->part2_3_length;
+	            best_noise_info = noise_info;
+	            *cod_info = cod_info_w;
+	            age = 0;
+	            /* save data so we can restore this quantization later */
+	            /*if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)*/{
+		            /* store for later reuse */
+		            memcpy(save_xrpow, xrpow, sizeof(FLOAT)*576);
+	            }
+            } else {
+                /* early stop? */
+                if (gfc->full_outer_loop == 0) {
+	                if (++age > search_limit && best_noise_info.over_count == 0)
+		                break;
+                    if ((gfc->noise_shaping_amp == 3) && bRefine &&
+                        age>30)
+                        break;
+                    if ((gfc->noise_shaping_amp == 3) && bRefine &&
+                        (cod_info_w.global_gain - best_ggain_pass1)>15)
+                        break;
+                }
+	        }
+        }
+        while ((cod_info_w.global_gain + cod_info_w.scalefac_scale) < 255);
+
+        if (gfc->noise_shaping_amp == 3) {
+            if (!bRefine) {
+                /* refine search*/
+                cod_info_w = *cod_info;
+		        memcpy( xrpow, save_xrpow, sizeof(FLOAT)*576);
+                age = 0;
+                best_ggain_pass1 = cod_info_w.global_gain;
+
+                bRefine = 1;
+            } else {
+                /* search already refined, stop*/
+                bEndOfSearch = 1;
+            }
+
+        } else {
+            bEndOfSearch = 1;
         }
-	}
     }
-    while (cod_info_w.global_gain < 255u);
 
-    assert (cod_info->global_gain < 256);
+    assert ((cod_info->global_gain + cod_info->scalefac_scale) <= 255);
     /*  finish up
      */
     if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)
 	/* restore for reuse on next try */
-	memcpy(xrpow, save_xrpow, sizeof(FLOAT8)*576);
+	memcpy(xrpow, save_xrpow, sizeof(FLOAT)*576);
     /*  do the 'substep shaping'
      */
     else if (gfc->substep_shaping & 1)
@@ -1277,15 +1442,15 @@ static void
 VBR_encode_granule (
     lame_global_flags	*gfp,
     gr_info		* const cod_info,
-    const FLOAT8	* const l3_xmin,     /* allowed distortion of the scalefactor */
-          FLOAT8                 xrpow[576],  /* coloured magnitudes of spectral values */
+    const FLOAT	* const l3_xmin,     /* allowed distortion of the scalefactor */
+          FLOAT                 xrpow[576],  /* coloured magnitudes of spectral values */
     const int                    ch, 
           int                    min_bits, 
           int                    max_bits )
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     gr_info         bst_cod_info;
-    FLOAT8          bst_xrpow [576]; 
+    FLOAT          bst_xrpow [576]; 
     int Max_bits  = max_bits;
     int real_bits = max_bits+1;
     int this_bits = (max_bits+min_bits)/2;
@@ -1321,7 +1486,7 @@ VBR_encode_granule (
             /*  store best quantization so far
              */
             bst_cod_info = *cod_info;
-            memcpy(bst_xrpow, xrpow, sizeof(FLOAT8)*576);
+            memcpy(bst_xrpow, xrpow, sizeof(FLOAT)*576);
 
             /*  try with fewer bits
              */
@@ -1341,7 +1506,7 @@ VBR_encode_granule (
                 /*  start again with best quantization so far
                  */
                 *cod_info = bst_cod_info;
-                memcpy(xrpow, bst_xrpow, sizeof(FLOAT8)*576);
+                memcpy(xrpow, bst_xrpow, sizeof(FLOAT)*576);
             }
         }
     } while (dbits>12);
@@ -1420,7 +1585,7 @@ calc_min_bits (
     lame_global_flags *gfp,
     const gr_info * const cod_info,
     const int             pe,
-    const FLOAT8          ms_ener_ratio, 
+    const FLOAT          ms_ener_ratio, 
     const int             bands,    
     const int             mch_bits,
     const int             analog_mean_bits,
@@ -1457,7 +1622,7 @@ calc_min_bits (
     if (gfc->mode_ext == MPG_MD_MS_LR && ch == 1) {
         /*  side channel will use a lower bit skeleton based on PE
          */ 
-        FLOAT8 fac  = .33 * (.5 - ms_ener_ratio) / .5;
+        FLOAT fac  = .33 * (.5 - ms_ener_ratio) / .5;
         min_pe_bits = (int)(min_pe_bits * ((1-fac)/(1+fac)));
     }
     min_pe_bits = Min (min_pe_bits, (1820 * gfp->out_samplerate / 44100));
@@ -1503,7 +1668,7 @@ VBR_prepare (
           FLOAT           pe            [2][2],
           FLOAT           ms_ener_ratio [2], 
           III_psy_ratio   ratio         [2][2], 
-          FLOAT8	  l3_xmin       [2][2][SFBMAX],
+          FLOAT	  l3_xmin       [2][2][SFBMAX],
           int             frameBits     [16],
           int            *analog_mean_bits,
           int            *min_mean_bits,
@@ -1578,7 +1743,7 @@ VBR_prepare (
 static void
 bitpressure_strategy(
     lame_internal_flags * gfc,
-    FLOAT8 l3_xmin[2][2][SFBMAX],
+    FLOAT l3_xmin[2][2][SFBMAX],
     int min_bits[2][2],  
     int max_bits[2][2] )  
 {
@@ -1586,7 +1751,7 @@ bitpressure_strategy(
     for (gr = 0; gr < gfc->mode_gr; gr++) {
         for (ch = 0; ch < gfc->channels_out; ch++) {
 	    gr_info *gi = &gfc->l3_side.tt[gr][ch];
-	    FLOAT8 *pxmin = l3_xmin[gr][ch];
+	    FLOAT *pxmin = l3_xmin[gr][ch];
 	    for (sfb = 0; sfb < gi->psy_lmax; sfb++) 
 		*pxmin++ *= 1.+.029*sfb*sfb/SBMAX_l/SBMAX_l;
 
@@ -1622,9 +1787,9 @@ VBR_iteration_loop (
     III_psy_ratio ratio[2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
-    FLOAT8 l3_xmin[2][2][SFBMAX];
+    FLOAT l3_xmin[2][2][SFBMAX];
   
-    FLOAT8    xrpow[576];
+    FLOAT    xrpow[576];
     int       bands[2][2];
     int       frameBits[15];
     int       save_bits[2][2];
@@ -1666,11 +1831,8 @@ VBR_iteration_loop (
             }
 
             if (gfp->VBR == vbr_mtrh) {
-                ret = VBR_noise_shaping (gfc, xrpow,
-					 min_bits[gr][ch], max_bits[gr][ch], 
-					 l3_xmin[gr][ch], gr, ch );
-                if (ret < 0)
-                    cod_info->part2_3_length = 100000;
+                VBR_noise_shaping (gfc, xrpow, l3_xmin[gr][ch],
+                                   max_bits[gr][ch], gr, ch );
             } 
             else
 	        VBR_encode_granule (gfp, cod_info, l3_xmin[gr][ch], xrpow,
@@ -1746,7 +1908,7 @@ calc_target_bits (
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     III_side_info_t *l3_side = &gfc->l3_side;
-    FLOAT8 res_factor;
+    FLOAT res_factor;
     int gr, ch, totbits, mean_bits;
     
     gfc->bitrate_index = gfc->VBR_max_bitrate;
@@ -1867,8 +2029,8 @@ ABR_iteration_loop(
     III_psy_ratio      ratio        [2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
-    FLOAT8    l3_xmin[SFBMAX];
-    FLOAT8    xrpow[576];
+    FLOAT    l3_xmin[SFBMAX];
+    FLOAT    xrpow[576];
     int       targ_bits[2][2];
     int       mean_bits, max_frame_bits;
     int       ch, gr, ath_over;
@@ -1888,8 +2050,21 @@ ABR_iteration_loop(
             ms_sparsing( gfc, gr );
         }
         for (ch = 0; ch < gfc->channels_out; ch++) {
+            FLOAT adjust, masking_lower_db;
             cod_info = &l3_side->tt[gr][ch];
 
+            if (cod_info->block_type == NORM_TYPE) {
+               /* adjust = 1.28/(1+exp(3.5-pe[gr][ch]/300.))-0.05;*/
+                adjust = 0;
+                masking_lower_db   = gfc->PSY->mask_adjust - adjust;
+            } else { 
+               /* adjust = 2.56/(1+exp(3.5-pe[gr][ch]/300.))-0.14;*/
+                adjust = 0;
+                masking_lower_db   = gfc->PSY->mask_adjust_short - adjust; 
+            }
+            gfc->masking_lower = pow (10.0, masking_lower_db * 0.1);
+      
+
             /*  cod_info, scalefac and xrpow get initialized in init_outer_loop
              */
             init_outer_loop(gfp, gfc, cod_info);
@@ -1928,7 +2103,7 @@ ABR_iteration_loop(
 
 /************************************************************************
  *
- *      iteration_loop()                                                    
+ *      CBR_iteration_loop()                                                    
  *
  *  author/date??
  *
@@ -1937,15 +2112,15 @@ ABR_iteration_loop(
  ************************************************************************/
 
 void 
-iteration_loop(
+CBR_iteration_loop(
     lame_global_flags *gfp, 
     FLOAT             pe           [2][2],
     FLOAT             ms_ener_ratio[2],
     III_psy_ratio      ratio        [2][2])
 {
     lame_internal_flags *gfc=gfp->internal_flags;
-    FLOAT8 l3_xmin[SFBMAX];
-    FLOAT8 xrpow[576];
+    FLOAT l3_xmin[SFBMAX];
+    FLOAT xrpow[576];
     int    targ_bits[2];
     int    mean_bits, max_bits;
     int    gr, ch;
@@ -1968,8 +2143,20 @@ iteration_loop(
         }
         
         for (ch=0 ; ch < gfc->channels_out ; ch ++) {
+            FLOAT adjust, masking_lower_db;
             cod_info = &l3_side->tt[gr][ch]; 
 
+            if (cod_info->block_type == NORM_TYPE) {
+               /* adjust = 1.28/(1+exp(3.5-pe[gr][ch]/300.))-0.05;*/
+                adjust = 0;
+                masking_lower_db   = gfc->PSY->mask_adjust - adjust;
+            } else { 
+               /* adjust = 2.56/(1+exp(3.5-pe[gr][ch]/300.))-0.14;*/
+                adjust = 0;
+                masking_lower_db   = gfc->PSY->mask_adjust_short - adjust; 
+            }
+            gfc->masking_lower = pow (10.0, masking_lower_db * 0.1);
+
             /*  init_outer_loop sets up cod_info, scalefac and xrpow 
              */
             init_outer_loop(gfp, gfc, cod_info);
