@@ -1,7 +1,10 @@
 /*
  *	quantize_pvt source file
  *
- *	Copyright (c) 1999 Takehiro TOMINAGA
+ *	Copyright (c) 1999-2002 Takehiro Tominaga
+ *	Copyright (c) 2000-2002 Robert Hegemann
+ *	Copyright (c) 2001 Naoki Shibata
+ *	Copyright (c) 2002-2005 Gabriel Bouvigne
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,7 +22,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: quantize_pvt.c,v 1.107 2003/12/22 09:44:10 bouvigne Exp $ */
+/* $Id: quantize_pvt.c,v 1.131.2.1 2005/11/20 14:08:25 bouvigne Exp $ */
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -139,15 +142,14 @@ const scalefac_struct sfBandIndex[9] =
 
 
 
-FLOAT8 pow20[Q_MAX+Q_MAX2];
-FLOAT8 ipow20[Q_MAX];
-FLOAT8 iipow20[Q_MAX2];
-FLOAT8 pow43[PRECALC_SIZE];
+FLOAT pow20[Q_MAX+Q_MAX2+1];
+FLOAT ipow20[Q_MAX];
+FLOAT pow43[PRECALC_SIZE];
 /* initialized in first call to iteration_init */
 #ifdef TAKEHIRO_IEEE754_HACK
-FLOAT8 adj43asm[PRECALC_SIZE];
+FLOAT adj43asm[PRECALC_SIZE];
 #else
-FLOAT8 adj43[PRECALC_SIZE];
+FLOAT adj43[PRECALC_SIZE];
 #endif
 
 /* 
@@ -284,7 +286,7 @@ static void compute_ath( lame_global_flags *gfp )
     gfc->ATH-> floor = 10. * log10( ATHmdct( gfp, -1. ) );
     
     /*
-    {   FLOAT8 g=10000, t=1e30, x;
+    {   FLOAT g=10000, t=1e30, x;
         for ( f = 100; f < 10000; f++ ) {
             x = ATHmdct( gfp, f );
             if ( t > x ) t = x, g = f;
@@ -294,6 +296,8 @@ static void compute_ath( lame_global_flags *gfp )
 }
 
 
+
+
 /************************************************************************/
 /*  initialization for iteration_loop */
 /************************************************************************/
@@ -312,7 +316,7 @@ iteration_init( lame_global_flags *gfp)
 
     pow43[0] = 0.0;
     for(i=1;i<PRECALC_SIZE;i++)
-        pow43[i] = pow((FLOAT8)i, 4.0/3.0);
+        pow43[i] = pow((FLOAT)i, 4.0/3.0);
 
 #ifdef TAKEHIRO_IEEE754_HACK
     adj43asm[0] = 0.0;
@@ -325,15 +329,15 @@ iteration_init( lame_global_flags *gfp)
 #endif
     for (i = 0; i < Q_MAX; i++)
 	ipow20[i] = pow(2.0, (double)(i - 210) * -0.1875);
-    for (i = 0; i < Q_MAX+Q_MAX2; i++)
+    for (i = 0; i <= Q_MAX+Q_MAX2; i++)
 	pow20[i] = pow(2.0, (double)(i - 210 - Q_MAX2) * 0.25);
-    for (i = 0; i < Q_MAX2; i++)
-        iipow20[i] = pow(2.0, (double)i * 0.1875);
 
     huffman_init(gfc);
+    quantize_init(gfc);
+    init_xrpow_core_init(gfc);
 
     if (gfp->psymodel == PSY_NSPSYTUNE) {
-	    FLOAT8 bass, alto, treble, sfb21;
+	    FLOAT bass, alto, treble, sfb21;
 
         i = (gfp->exp_nspsytune >> 2) & 63;
         if (i >= 32)
@@ -358,24 +362,20 @@ iteration_init( lame_global_flags *gfp)
         sfb21 = treble * pow(10, i / 4.0 / 10.0);
 
 	for (i = 0; i < SBMAX_l; i++) {
-	    FLOAT8 f;
+	    FLOAT f;
 	    if      (i <=  6) f = bass;
 	    else if (i <= 13) f = alto;
 	    else if (i <= 20) f = treble;
 	    else              f = sfb21;
-	    if ((gfp->VBR == vbr_off || gfp->VBR == vbr_abr) && gfp->quality <= 1)
-		f *= 0.001;
 
 	    gfc->nsPsy.longfact[i] = f;
 	}
 	for (i = 0; i < SBMAX_s; i++) {
-	    FLOAT8 f;
+	    FLOAT f;
 	    if      (i <=  5) f = bass;
 	    else if (i <= 10) f = alto;
-	    else              f = treble;
-
-	    if ((gfp->VBR == vbr_off || gfp->VBR == vbr_abr) && gfp->quality <= 1)
-		f *= 0.001;
+	    else if (i <= 11) f = treble;
+	    else              f = sfb21;
 
 	    gfc->nsPsy.shortfact[i] = f;
 	}
@@ -462,7 +462,7 @@ int on_pe( lame_global_flags *gfp, FLOAT pe[][2], III_side_info_t *l3_side,
 
 
 
-void reduce_side(int targ_bits[2],FLOAT8 ms_ener_ratio,int mean_bits,int max_bits)
+void reduce_side(int targ_bits[2],FLOAT ms_ener_ratio,int mean_bits,int max_bits)
 {
   int move_bits;
   FLOAT fac;
@@ -551,17 +551,17 @@ int calc_xmin(
         lame_global_flags *gfp,
         const III_psy_ratio * const ratio,
 	    gr_info       *  const cod_info, 
-	    FLOAT8 * pxmin
+	    FLOAT * pxmin
     )
 {
     lame_internal_flags *gfc = gfp->internal_flags;
     int sfb, gsfb, j=0, ath_over=0, k;
     ATH_t * ATH = gfc->ATH;
-    const FLOAT8 *xr = cod_info->xr;
+    const FLOAT *xr = cod_info->xr;
     int max_nonzero;
 
     for (gsfb = 0; gsfb < cod_info->psy_lmax; gsfb++) {
-	FLOAT8 en0, xmin;
+	FLOAT en0, xmin;
 	int width, l;
 	if (gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh)
 	    xmin = athAdjust(ATH->adjust, ATH->l[gsfb], ATH->floor);
@@ -578,7 +578,7 @@ int calc_xmin(
 	if (en0 > xmin) ath_over++;
 
 	if (!gfp->ATHonly) {
-	    FLOAT8 x = ratio->en.l[gsfb];
+	    FLOAT x = ratio->en.l[gsfb];
 	    if (x > 0.0) {
 		x = en0 * ratio->thm.l[gsfb] * gfc->masking_lower / x;
 		if (xmin < x)
@@ -605,7 +605,7 @@ int calc_xmin(
 
     for (sfb = cod_info->sfb_smin; gsfb < cod_info->psymax; sfb++, gsfb += 3) {
 	int width, b;
-	FLOAT8 tmpATH;
+	FLOAT tmpATH;
 	if ( gfp->VBR == vbr_rh || gfp->VBR == vbr_mtrh )
 	    tmpATH = athAdjust( ATH->adjust, ATH->s[sfb], ATH->floor );
 	else
@@ -613,7 +613,7 @@ int calc_xmin(
 
 	width = cod_info->width[gsfb];
 	for ( b = 0; b < 3; b++ ) {
-	    FLOAT8 en0 = 0.0, xmin;
+	    FLOAT en0 = 0.0, xmin;
 	    int l = width >> 1;
 	    do {
 		en0 += xr[j] * xr[j]; j++;
@@ -623,7 +623,7 @@ int calc_xmin(
 
 	    xmin = tmpATH;
 	    if (!gfp->ATHonly && !gfp->ATHshort) {
-		FLOAT8 x = ratio->en.s[sfb][b];
+		FLOAT x = ratio->en.s[sfb][b];
 		if (x > 0.0)
 		    x = en0 * ratio->thm.s[sfb][b] * gfc->masking_lower / x;
 		if (xmin < x) 
@@ -643,17 +643,55 @@ int calc_xmin(
 }
 
 
+FLOAT calc_noise_core_c( const gr_info * const cod_info, 
+                        int *startline,
+                        int l,
+                        FLOAT step)
+{
+    FLOAT noise = 0;
+    int j = *startline;
+    const int *ix = cod_info->l3_enc;
 
+    if (j> cod_info->count1) {
+	    while (l--) {
+            FLOAT temp;
+            temp = cod_info->xr[j];j++;
+            noise += temp * temp;
+            temp = cod_info->xr[j];j++;
+            noise += temp * temp;
+ 	    }
+    } else if (j> cod_info->big_values) {
+        FLOAT ix01[2];
+        ix01[0] = 0;
+        ix01[1] = step;
+	    while (l--) {
+            FLOAT temp;
+            temp = fabs(cod_info->xr[j]) - ix01[ix[j]];j++;
+            noise += temp * temp;
+            temp = fabs(cod_info->xr[j]) - ix01[ix[j]];j++;
+            noise += temp * temp;
+        }
+    } else {
+	    while (l--) {
+            FLOAT temp;
+            temp = fabs(cod_info->xr[j]) - pow43[ix[j]] * step;j++;
+	        noise += temp * temp;
+	        temp = fabs(cod_info->xr[j]) - pow43[ix[j]] * step;j++;
+	        noise += temp * temp;
+ 	    }
+    }
 
-
-
+    *startline = j;
+    return noise;
+}
+ 
 
 /*************************************************************************/
 /*            calc_noise                                                 */
 /*************************************************************************/
 
 /* -oo dB  =>  -1.00 */
-/* - 6 dB  =>  -0.97 */
+/* - 6 dB  =>  -0.97 */ 
 /* - 3 dB  =>  -0.80 */
 /* - 2 dB  =>  -0.64 */
 /* - 1 dB  =>  -0.38 */
@@ -667,18 +705,21 @@ int calc_xmin(
 int  calc_noise( 
         const lame_internal_flags * const gfc,
         const gr_info             * const cod_info,
-        const FLOAT8              * l3_xmin, 
-              FLOAT8              * distort,
+        const FLOAT              * l3_xmin, 
+              FLOAT              * distort,
               calc_noise_result   * const res,
               calc_noise_data * prev_noise)
 {
     int sfb, l, over=0;
-    FLOAT8 over_noise_db = 0;
-    FLOAT8 tot_noise_db  = 0; /*    0 dB relative to masking */
-    FLOAT8 max_noise = -20.0; /* -200 dB relative to masking */
+    FLOAT over_noise_db = 0;
+    FLOAT tot_noise_db  = 0; /*    0 dB relative to masking */
+    FLOAT max_noise = -20.0; /* -200 dB relative to masking */
     int j = 0;
     const int *ix = cod_info->l3_enc;
     const int *scalefac = cod_info->scalefac;
+    FLOAT sfb_noise[39];
+   
+    res->over_SSD = 0;
 
 
     for (sfb = 0; sfb < cod_info->psymax; sfb++) {
@@ -687,10 +728,9 @@ int  calc_noise(
 	        - (((*scalefac++) + (cod_info->preflag ? pretab[sfb] : 0))
 	           << (cod_info->scalefac_scale + 1))
 	        - cod_info->subblock_gain[cod_info->window[sfb]] * 8;
-	    FLOAT8 step = POW20(s);
-	    FLOAT8 noise = 0.0;
+	    FLOAT noise = 0.0;
 
-        if (prev_noise && (prev_noise->step[sfb] == step)){
+        if (prev_noise && (prev_noise->step[sfb] == s)){
 
             /* use previously computed values */
             noise = prev_noise->noise[sfb];
@@ -700,25 +740,25 @@ int  calc_noise(
 	        noise = prev_noise->noise_log[sfb];
 
         } else {
+	        FLOAT step = POW20(s);
             l = cod_info->width[sfb] >> 1;
 
             if ((j+cod_info->width[sfb])>cod_info->max_nonzero_coeff) {
                 int usefullsize;
                 usefullsize = cod_info->max_nonzero_coeff - j +1;
-                l = usefullsize >> 1;
+
+                if (usefullsize > 0)
+                    l = usefullsize >> 1;
+                else 
+                    l = 0;
             }
 
-	        do {
-                FLOAT8 temp;
-                temp = fabs(cod_info->xr[j]) - pow43[ix[j]] * step;j++;
-	            noise += temp * temp;
-	            temp = fabs(cod_info->xr[j]) - pow43[ix[j]] * step;j++;
-	            noise += temp * temp;
- 	        } while (--l > 0);
+            noise = calc_noise_core_c(cod_info, &j, l, step); 
+
 
             if (prev_noise) {
                 /* save noise values */
-                prev_noise->step[sfb] = step;
+                prev_noise->step[sfb] = s;
                 prev_noise->noise[sfb] = noise;
             }
 
@@ -733,18 +773,29 @@ int  calc_noise(
             }
         }
 
+        if (prev_noise) {
+            /* save noise values */
+            prev_noise->global_gain = cod_info->global_gain;;
+        }
 
 
 	    /*tot_noise *= Max(noise, 1E-20); */
 	    tot_noise_db += noise;
 
         if (noise > 0.0) {
+            int tmp;
+            
+            tmp = Max((int)(noise*10 + .5), 1);
+            res->over_SSD += tmp*tmp;
+
 	        over++;
 	        /* multiplying here is adding in dB -but can overflow */
 	        /*over_noise *= noise; */
 	        over_noise_db += noise;
 	    }
 	    max_noise=Max(max_noise,noise);
+
+        sfb_noise[sfb] = noise;
     }
 
     res->over_count = over;
@@ -761,13 +812,6 @@ int  calc_noise(
 
 
 
-
-
-
-
-
-
-
 #ifdef HAVE_GTK
 /************************************************************************
  *
@@ -792,11 +836,11 @@ void set_pinfo (
     lame_internal_flags *gfc=gfp->internal_flags;
     int sfb, sfb2;
     int j,i,l,start,end,bw;
-    FLOAT8 en0,en1;
+    FLOAT en0,en1;
     FLOAT ifqstep = ( cod_info->scalefac_scale == 0 ) ? .5 : 1.0;
     int* scalefac = cod_info->scalefac;
 
-    FLOAT8 l3_xmin[SFBMAX], xfsf[SFBMAX];
+    FLOAT l3_xmin[SFBMAX], xfsf[SFBMAX];
     calc_noise_result noise;
 
     calc_xmin (gfp, ratio, cod_info, l3_xmin);
@@ -883,6 +927,7 @@ void set_pinfo (
     gfc->pinfo->max_noise [gr][ch] = noise.max_noise * 10.0;
     gfc->pinfo->over_noise[gr][ch] = noise.over_noise * 10.0;
     gfc->pinfo->tot_noise [gr][ch] = noise.tot_noise * 10.0;
+    gfc->pinfo->over_SSD [gr][ch] = noise.over_SSD;
 }
 
 
