@@ -1,7 +1,8 @@
 /*
  *	MP3 huffman table selecting and bit counting
  *
- *	Copyright (c) 1999 Takehiro TOMINAGA
+ *	Copyright (c) 1999-2005 Takehiro TOMINAGA
+ *	Copyright (c) 2002-2005 Gabriel Bouvigne
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: takehiro.c,v 1.37 2004/01/12 08:41:33 bouvigne Exp $ */
+/* $Id: takehiro.c,v 1.61.2.1 2005/11/20 14:08:25 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -86,6 +87,23 @@ static const struct
  *********************************************************************/
 
 
+
+
+
+void quantize_lines_xrpow_01(int l, FLOAT istep, const FLOAT* xr, int* ix)
+{
+    const FLOAT compareval0 = (1.0 - 0.4054)/istep;
+
+    assert (l>0);
+    l= l>>1;
+    while (l--) {
+        *(ix++) = (compareval0 > *xr++) ? 0 : 1;
+        *(ix++) = (compareval0 > *xr++) ? 0 : 1;
+    }
+}
+
+
+
 #ifdef TAKEHIRO_IEEE754_HACK
 
 typedef union {
@@ -97,189 +115,99 @@ typedef union {
 #define MAGIC_INT 0x4b000000
 
 
-static void quantize_xrpow(const FLOAT8 *xp, int *pi, FLOAT8 istep, gr_info * const cod_info, calc_noise_data* prev_noise)
+void quantize_lines_xrpow(int l, FLOAT istep, const FLOAT* xp, int* pi)
 {
-    /* quantize on xr^(3/4) instead of xr */
     fi_union *fi;
-    int sfb;
-    int sfbmax;
-    int j=0;
-    int prev_data_use;
+    int remaining;
 
-    /* bad way to check if global_gain is unchanged
-    since last call.
-    It might also be possible to re-use previous data
-    even when global_gain is changed */
-    prev_data_use = (prev_noise && 
-                    (cod_info->block_type != SHORT_TYPE) && 
-                    (POW20(cod_info->global_gain) == prev_noise->step[21]));
+    assert (l>0);
 
     fi = (fi_union *)pi;
 
-    if (cod_info->block_type == SHORT_TYPE)
-        sfbmax = 38;
-    else 
-        sfbmax = 21;
+    l = l>>1;
+    remaining = l%2;
+    l = l>>1;
+    while (l--) {
+	    double x0 = istep * xp[0];
+	    double x1 = istep * xp[1];
+	    double x2 = istep * xp[2];
+	    double x3 = istep * xp[3];
 
-    for (sfb = 0; sfb <= sfbmax; sfb++) {
-	    FLOAT8 step;
+	    x0 += MAGIC_FLOAT; fi[0].f = x0;
+	    x1 += MAGIC_FLOAT; fi[1].f = x1;
+	    x2 += MAGIC_FLOAT; fi[2].f = x2;
+	    x3 += MAGIC_FLOAT; fi[3].f = x3;
 
-        if (prev_data_use) {
-            int s =
-	            cod_info->global_gain
-	            - ((cod_info->scalefac[sfb] + (cod_info->preflag ? pretab[sfb] : 0))
-	               << (cod_info->scalefac_scale + 1))
-	            - cod_info->subblock_gain[cod_info->window[sfb]] * 8;
-	        step = POW20(s);
-        }
-        
-        if (prev_data_use && (prev_noise->step[sfb] == step)){
-            /* do not recompute this part*/
-            fi += cod_info->width[sfb];
-            xp += cod_info->width[sfb];
-        } else {
-            int l;
-            int remaining;
+	    fi[0].f = x0 + (adj43asm - MAGIC_INT)[fi[0].i];
+	    fi[1].f = x1 + (adj43asm - MAGIC_INT)[fi[1].i];
+	    fi[2].f = x2 + (adj43asm - MAGIC_INT)[fi[2].i];
+	    fi[3].f = x3 + (adj43asm - MAGIC_INT)[fi[3].i];
 
-            l = cod_info->width[sfb] >> 1;
+	    fi[0].i -= MAGIC_INT;
+	    fi[1].i -= MAGIC_INT;
+	    fi[2].i -= MAGIC_INT;
+	    fi[3].i -= MAGIC_INT;
+	    fi += 4;
+	    xp += 4;
+    };
+    if (remaining) {
+	    double x0 = istep * xp[0];
+	    double x1 = istep * xp[1];
 
-            if ((j+cod_info->width[sfb])>cod_info->max_nonzero_coeff) {
-                int usefullsize;
-                usefullsize = cod_info->max_nonzero_coeff - j +1;
-                memset(&pi[cod_info->max_nonzero_coeff],0,
-                    sizeof(int)*(575-cod_info->max_nonzero_coeff));
-                l = usefullsize >> 1;
-            }
+	    x0 += MAGIC_FLOAT; fi[0].f = x0;
+	    x1 += MAGIC_FLOAT; fi[1].f = x1;
 
-            remaining = l%2;
-            l = l>>1;
-            do {
-	            double x0 = istep * xp[0];
-	            double x1 = istep * xp[1];
-	            double x2 = istep * xp[2];
-	            double x3 = istep * xp[3];
+	    fi[0].f = x0 + (adj43asm - MAGIC_INT)[fi[0].i];
+	    fi[1].f = x1 + (adj43asm - MAGIC_INT)[fi[1].i];
 
-	            x0 += MAGIC_FLOAT; fi[0].f = x0;
-	            x1 += MAGIC_FLOAT; fi[1].f = x1;
-	            x2 += MAGIC_FLOAT; fi[2].f = x2;
-	            x3 += MAGIC_FLOAT; fi[3].f = x3;
-
-	            fi[0].f = x0 + (adj43asm - MAGIC_INT)[fi[0].i];
-	            fi[1].f = x1 + (adj43asm - MAGIC_INT)[fi[1].i];
-	            fi[2].f = x2 + (adj43asm - MAGIC_INT)[fi[2].i];
-	            fi[3].f = x3 + (adj43asm - MAGIC_INT)[fi[3].i];
-
-	            fi[0].i -= MAGIC_INT;
-	            fi[1].i -= MAGIC_INT;
-	            fi[2].i -= MAGIC_INT;
-	            fi[3].i -= MAGIC_INT;
-	            fi += 4;
-	            xp += 4;
-            } while (--l > 0);
-            if (remaining) {
-	            double x0 = istep * xp[0];
-	            double x1 = istep * xp[1];
-
-	            x0 += MAGIC_FLOAT; fi[0].f = x0;
-	            x1 += MAGIC_FLOAT; fi[1].f = x1;
-
-	            fi[0].f = x0 + (adj43asm - MAGIC_INT)[fi[0].i];
-	            fi[1].f = x1 + (adj43asm - MAGIC_INT)[fi[1].i];
-
-	            fi[0].i -= MAGIC_INT;
-	            fi[1].i -= MAGIC_INT;
-	            fi += 2;
-	            xp += 2;
-            }
-        }
-        j += cod_info->width[sfb];
+	    fi[0].i -= MAGIC_INT;
+	    fi[1].i -= MAGIC_INT;
     }
+
 }
 
+
 #  define ROUNDFAC -0.0946
-static void quantize_xrpow_ISO(const FLOAT8 *xp, int *pi, FLOAT8 istep, gr_info * const cod_info, calc_noise_data* prev_noise)
+void quantize_lines_xrpow_ISO(int l, FLOAT istep, const FLOAT* xp, int* pi)
 {
-    /* quantize on xr^(3/4) instead of xr */
     fi_union *fi;
-    int sfb;
-    int sfbmax;
-    int j=0;
-    int prev_data_use;
+    int remaining;
 
-    /* bad way to check if global_gain is unchanged
-    since last call.
-    It might also be possible to re-use previous data
-    even when global_gain is changed */
-    prev_data_use = (prev_noise && 
-                    (cod_info->block_type != SHORT_TYPE) && 
-                    (POW20(cod_info->global_gain) == prev_noise->step[21]));
+    assert (l>0);
 
     fi = (fi_union *)pi;
 
-    if (cod_info->block_type == SHORT_TYPE)
-        sfbmax = 38;
-    else 
-        sfbmax = 21;
+    l = l>>1;
+    remaining = l%2;
+    l = l>>1;
+    while (l--) {
+	        fi[0].f = istep * xp[0] + (ROUNDFAC + MAGIC_FLOAT);
+	        fi[1].f = istep * xp[1] + (ROUNDFAC + MAGIC_FLOAT);
+	        fi[2].f = istep * xp[2] + (ROUNDFAC + MAGIC_FLOAT);
+	        fi[3].f = istep * xp[3] + (ROUNDFAC + MAGIC_FLOAT);
 
-    for (sfb = 0; sfb <= sfbmax; sfb++) {
-	    FLOAT8 step;
+	        fi[0].i -= MAGIC_INT;
+	        fi[1].i -= MAGIC_INT;
+	        fi[2].i -= MAGIC_INT;
+	        fi[3].i -= MAGIC_INT;
+	        fi+=4;
+	        xp+=4;
+    };
+    if (remaining) {
+	        fi[0].f = istep * xp[0] + (ROUNDFAC + MAGIC_FLOAT);
+	        fi[1].f = istep * xp[1] + (ROUNDFAC + MAGIC_FLOAT);
 
-        if (prev_data_use) {
-            int s =
-	            cod_info->global_gain
-	            - ((cod_info->scalefac[sfb] + (cod_info->preflag ? pretab[sfb] : 0))
-	               << (cod_info->scalefac_scale + 1))
-	            - cod_info->subblock_gain[cod_info->window[sfb]] * 8;
-	        step = POW20(s);
-        }
-        
-        if (prev_data_use && (prev_noise->step[sfb] == step)){
-            /* do not recompute this part*/
-            fi += cod_info->width[sfb];
-            xp += cod_info->width[sfb];
-        } else {
-            int l;
-            int remaining;
-
-            l = cod_info->width[sfb] >> 1;
-
-            if ((j+cod_info->width[sfb])>cod_info->max_nonzero_coeff) {
-                int usefullsize;
-                usefullsize = cod_info->max_nonzero_coeff - j +1;
-                memset(&pi[cod_info->max_nonzero_coeff],0,
-                    sizeof(int)*(575-cod_info->max_nonzero_coeff));
-                l = usefullsize >> 1;
-            }
-
-            remaining = l%2;
-            l = l>>1;
-            do {
-	            fi[0].f = istep * xp[0] + (ROUNDFAC + MAGIC_FLOAT);
-	            fi[1].f = istep * xp[1] + (ROUNDFAC + MAGIC_FLOAT);
-	            fi[2].f = istep * xp[2] + (ROUNDFAC + MAGIC_FLOAT);
-	            fi[3].f = istep * xp[3] + (ROUNDFAC + MAGIC_FLOAT);
-
-	            fi[0].i -= MAGIC_INT;
-	            fi[1].i -= MAGIC_INT;
-	            fi[2].i -= MAGIC_INT;
-	            fi[3].i -= MAGIC_INT;
-	            fi+=4;
-	            xp+=4;
-            } while (--l > 0);
-            if (remaining) {
-	            fi[0].f = istep * xp[0] + (ROUNDFAC + MAGIC_FLOAT);
-	            fi[1].f = istep * xp[1] + (ROUNDFAC + MAGIC_FLOAT);
-
-	            fi[0].i -= MAGIC_INT;
-	            fi[1].i -= MAGIC_INT;
-	            fi+=2;
-	            xp+=2;
-            }
-        }
-        j += cod_info->width[sfb];
+	        fi[0].i -= MAGIC_INT;
+	        fi[1].i -= MAGIC_INT;
     }
+
 }
 
+
+
+
+
+
 #else
 
 /*********************************************************************
@@ -299,199 +227,254 @@ static void quantize_xrpow_ISO(const FLOAT8 *xp, int *pi, FLOAT8 istep, gr_info
 #define ROUNDFAC 0.4054
 
 
-
-static void quantize_xrpow(const FLOAT8 *xr, int *ix, FLOAT8 istep, gr_info * const cod_info, calc_noise_data* prev_noise)
+void quantize_lines_xrpow(int l, FLOAT istep, const FLOAT* xr, int* ix)
 {
-    /* quantize on xr^(3/4) instead of xr */
-    int sfb;
-    int sfbmax;
-    int j=0;
-    int prev_data_use;
-    int *out;
-    out = ix;
+    int remaining;
 
-    /* bad way to check if global_gain is unchanged
-    since last call.
-    It might also be possible to re-use previous data
-    even when global_gain is changed */
-    prev_data_use = (prev_noise && 
-                    (cod_info->block_type != SHORT_TYPE) && 
-                    (POW20(cod_info->global_gain) == prev_noise->step[21]));
+    assert (l>0);
 
+    l = l>>1;
+    remaining = l%2;
+    l = l>>1;
+    while (l--) {
+	    FLOAT	x0, x1, x2, x3;
+	    int	rx0, rx1, rx2, rx3;
 
-    if (cod_info->block_type == SHORT_TYPE)
-        sfbmax = 38;
-    else 
-        sfbmax = 21;
+        x0 = *xr++ * istep;
+	    x1 = *xr++ * istep;
+	    XRPOW_FTOI(x0, rx0);
+	    x2 = *xr++ * istep;
+	    XRPOW_FTOI(x1, rx1);
+	    x3 = *xr++ * istep;
+	    XRPOW_FTOI(x2, rx2);
+	    x0 += QUANTFAC(rx0);
+	    XRPOW_FTOI(x3, rx3);
+	    x1 += QUANTFAC(rx1);
+	    XRPOW_FTOI(x0,*ix++);
+	    x2 += QUANTFAC(rx2);
+	    XRPOW_FTOI(x1,*ix++);
+	    x3 += QUANTFAC(rx3);
+	    XRPOW_FTOI(x2,*ix++);
+	    XRPOW_FTOI(x3,*ix++);
+    };
+    if (remaining) {
+	    FLOAT	x0, x1;
+	    int	rx0, rx1;
 
-    for (sfb = 0; sfb <= sfbmax; sfb++) {
-	    FLOAT8 step;
-
-        if (prev_data_use) {
-            int s =
-	            cod_info->global_gain
-	            - ((cod_info->scalefac[sfb] + (cod_info->preflag ? pretab[sfb] : 0))
-	               << (cod_info->scalefac_scale + 1))
-	            - cod_info->subblock_gain[cod_info->window[sfb]] * 8;
-	        step = POW20(s);
-        }
-        
-        if (prev_data_use && (prev_noise->step[sfb] == step)){
-            /* do not recompute this part */
-            xr += cod_info->width[sfb];
-            ix += cod_info->width[sfb];
-        } else {
-            int l;
-            int remaining;
-
-            l = cod_info->width[sfb] >> 1;
-
-            if ((j+cod_info->width[sfb])>cod_info->max_nonzero_coeff) {
-                int usefullsize;
-                usefullsize = cod_info->max_nonzero_coeff - j +1;
-                memset(&out[cod_info->max_nonzero_coeff],0,
-                    sizeof(int)*(575-cod_info->max_nonzero_coeff));
-                l = usefullsize >> 1;
-            }
-
-            remaining = l%2;
-            l = l>>1;
-            do {
-	            FLOAT8	x0, x1, x2, x3;
-	            int	rx0, rx1, rx2, rx3;
-
-                x0 = *xr++ * istep;
-	            x1 = *xr++ * istep;
-	            XRPOW_FTOI(x0, rx0);
-	            x2 = *xr++ * istep;
-	            XRPOW_FTOI(x1, rx1);
-	            x3 = *xr++ * istep;
-	            XRPOW_FTOI(x2, rx2);
-	            x0 += QUANTFAC(rx0);
-	            XRPOW_FTOI(x3, rx3);
-	            x1 += QUANTFAC(rx1);
-	            XRPOW_FTOI(x0,*ix++);
-	            x2 += QUANTFAC(rx2);
-	            XRPOW_FTOI(x1,*ix++);
-	            x3 += QUANTFAC(rx3);
-	            XRPOW_FTOI(x2,*ix++);
-	            XRPOW_FTOI(x3,*ix++);
-            } while (--l > 0);
-            if (remaining) {
-	            FLOAT8	x0, x1;
-	            int	rx0, rx1;
-
-                x0 = *xr++ * istep;
-	            x1 = *xr++ * istep;
-	            XRPOW_FTOI(x0, rx0);
-	            XRPOW_FTOI(x1, rx1);
-	            x0 += QUANTFAC(rx0);
-	            x1 += QUANTFAC(rx1);
-	            XRPOW_FTOI(x0,*ix++);
-	            XRPOW_FTOI(x1,*ix++);
-            }
-        }
-        j += cod_info->width[sfb];
+        x0 = *xr++ * istep;
+	    x1 = *xr++ * istep;
+	    XRPOW_FTOI(x0, rx0);
+	    XRPOW_FTOI(x1, rx1);
+	    x0 += QUANTFAC(rx0);
+	    x1 += QUANTFAC(rx1);
+	    XRPOW_FTOI(x0,*ix++);
+	    XRPOW_FTOI(x1,*ix++);
     }
+
 }
 
 
 
-
-
-
-static void quantize_xrpow_ISO(const FLOAT8 *xr, int *ix, FLOAT8 istep, gr_info * const cod_info, calc_noise_data* prev_noise)
+void quantize_lines_xrpow_ISO(int l, FLOAT istep, const FLOAT* xr, int* ix)
 {
-    /* quantize on xr^(3/4) instead of xr */
-    int sfb;
-    int sfbmax;
-    int j=0;
-    int prev_data_use;
-    int *out;
-    const FLOAT8 compareval0 = (1.0 - 0.4054)/istep;
-    const FLOAT8 compareval1 = (2.0 - 0.4054)/istep;
-    out = ix;
 
-    /* bad way to check if global_gain is unchanged
-    since last call.
-    It might also be possible to re-use previous data
-    even when global_gain is changed */
-    prev_data_use = (prev_noise && 
-                    (cod_info->block_type != SHORT_TYPE) && 
-                    (POW20(cod_info->global_gain) == prev_noise->step[21]));
+    const FLOAT compareval0 = (1.0 - 0.4054)/istep;
+    const FLOAT compareval1 = (2.0 - 0.4054)/istep;
 
+    assert (l>0);
 
-    if (cod_info->block_type == SHORT_TYPE)
-        sfbmax = 38;
-    else 
-        sfbmax = 21;
+    /* depending on architecture, it may be worth calculating a few more
+    compareval's.
 
-    for (sfb = 0; sfb <= sfbmax; sfb++) {
-	    FLOAT8 step;
+    eg.  compareval1 = (2.0 - 0.4054)/istep;
+    .. and then after the first compare do this ...
+    if compareval1>*xr then ix = 1;
 
-        if (prev_data_use) {
-            int s =
-	            cod_info->global_gain
-	            - ((cod_info->scalefac[sfb] + (cod_info->preflag ? pretab[sfb] : 0))
-	               << (cod_info->scalefac_scale + 1))
-	            - cod_info->subblock_gain[cod_info->window[sfb]] * 8;
-	        step = POW20(s);
-        }
-        
-        if (prev_data_use && (prev_noise->step[sfb] == step)){
-            /* do not recompute this part */
-            xr += cod_info->width[sfb];
-            ix += cod_info->width[sfb];
+    On a pentium166, it's only worth doing the one compare (as done here),
+    as the second compare becomes more expensive than just calculating
+    the value. Architectures with slow FP operations may want to add some
+    more comparevals. try it and send your diffs statistically speaking
+
+    73% of all xr*istep values give ix=0
+    16% will give 1
+    4%  will give 2
+    */
+    while (l--) {
+        if (compareval0 > *xr) {
+            *(ix++) = 0;
+            xr++;
+        } else if (compareval1 > *xr) {
+            *(ix++) = 1;
+            xr++;
         } else {
-            int l;
-            l = cod_info->width[sfb];
-
-            if ((j+cod_info->width[sfb])>cod_info->max_nonzero_coeff) {
-                int usefullsize;
-                usefullsize = cod_info->max_nonzero_coeff - j +1;
-                memset(&out[cod_info->max_nonzero_coeff],0,
-                    sizeof(int)*(575-cod_info->max_nonzero_coeff));
-                l = usefullsize;
-            }
-
-            do {
-                /* depending on architecture, it may be worth calculating a few more
-                   compareval's.
-
-                   eg.  compareval1 = (2.0 - 0.4054)/istep;
-                   .. and then after the first compare do this ...
-                   if compareval1>*xr then ix = 1;
-
-                   On a pentium166, it's only worth doing the one compare (as done here),
-                   as the second compare becomes more expensive than just calculating
-                   the value. Architectures with slow FP operations may want to add some
-                   more comparevals. try it and send your diffs statistically speaking
-
-                   73% of all xr*istep values give ix=0
-                   16% will give 1
-                   4%  will give 2
-                */
-	            if (compareval0 > *xr) {
-	                *(ix++) = 0;
-	                xr++;
-	            } else if (compareval1 > *xr) {
-	                *(ix++) = 1;
-	                xr++;
-	            } else {
-	                /*    *(ix++) = (int)( istep*(*(xr++))  + 0.4054); */
-	                XRPOW_FTOI(  istep*(*(xr++))  + ROUNDFAC , *(ix++) );
-	            }
-            } while (--l > 0);
+            /*    *(ix++) = (int)( istep*(*(xr++))  + 0.4054); */
+            XRPOW_FTOI(  istep*(*(xr++))  + ROUNDFAC , *(ix++) );
         }
-        j += cod_info->width[sfb];
     }
+
 }
 
 
 
+
 #endif
 
 
+
+/*********************************************************************
+ * Quantization function
+ * This function will select which lines to quantize and call the
+ * proper quantization function
+ *********************************************************************/
+
+static void quantize_xrpow(const FLOAT *xp, int *pi, FLOAT istep, gr_info * const cod_info, calc_noise_data* prev_noise, lame_internal_flags * const gfc)
+{
+    /* quantize on xr^(3/4) instead of xr */
+    int sfb;
+    int sfbmax;
+    int j=0;
+    int prev_data_use;
+    int *iData;
+    int accumulate=0;
+    int accumulate01=0;
+    int *acc_iData;
+    const FLOAT *acc_xp;
+
+    iData = pi;
+    acc_xp = xp;
+    acc_iData = iData;
+
+
+    /* Reusing previously computed data does not seems to work if global gain
+    is changed. Finding why it behaves this way would allow to use a cache of 
+    previously computed values (let's 10 cached values per sfb) that would 
+    probably provide a noticeable speedup*/
+    prev_data_use = (prev_noise  && 
+                    (cod_info->global_gain == prev_noise->global_gain));
+
+    if (cod_info->block_type == SHORT_TYPE)
+        sfbmax = 38;
+    else 
+        sfbmax = 21;
+
+    for (sfb = 0; sfb <= sfbmax; sfb++) {
+	    int step = -1;
+
+        if (prev_data_use || cod_info->block_type == NORM_TYPE) {
+            step =
+	            cod_info->global_gain
+	            - ((cod_info->scalefac[sfb] + (cod_info->preflag ? pretab[sfb] : 0))
+	               << (cod_info->scalefac_scale + 1))
+	            - cod_info->subblock_gain[cod_info->window[sfb]] * 8;
+        }
+        assert( cod_info->width[sfb] >= 0 );
+        if (prev_data_use && (prev_noise->step[sfb] == step)){
+            /* do not recompute this part,
+               but compute accumulated lines */
+            if (accumulate) {
+                gfc->quantize_lines_xrpow(accumulate, istep, acc_xp, acc_iData);
+                accumulate = 0;
+            }
+            if (accumulate01) {
+                quantize_lines_xrpow_01(accumulate01, istep, acc_xp, acc_iData);
+                accumulate01 = 0;
+            }
+        } else { /*should compute this part*/
+            int l;
+            l = cod_info->width[sfb];
+
+            if ((j+cod_info->width[sfb])>cod_info->max_nonzero_coeff) {
+                /*do not compute upper zero part*/
+                int usefullsize;
+                usefullsize = cod_info->max_nonzero_coeff - j +1;
+                memset(&pi[cod_info->max_nonzero_coeff],0,
+                    sizeof(int)*(576-cod_info->max_nonzero_coeff));
+                l = usefullsize;
+
+                if (l<0) {
+                    l = 0;
+                }
+
+                /* no need to compute higher sfb values */
+                sfb = sfbmax + 1;
+            }
+
+            /*accumulate lines to quantize*/
+            if (!accumulate && !accumulate01) {
+                acc_iData = iData;
+                acc_xp = xp;
+            }
+            if (prev_noise &&
+                prev_noise->sfb_count1 > 0 &&
+                sfb >= prev_noise->sfb_count1 &&
+                prev_noise->step[sfb] > 0 &&
+                step >= prev_noise->step[sfb]) {
+
+                if (accumulate) {
+                    gfc->quantize_lines_xrpow(accumulate, istep, acc_xp, acc_iData);
+                    accumulate = 0;
+                    acc_iData = iData;
+                    acc_xp = xp;
+                }
+                accumulate01 += l;
+            } else {
+                if (accumulate01) {
+                    quantize_lines_xrpow_01(accumulate01, istep, acc_xp, acc_iData);
+                    accumulate01 = 0;
+                    acc_iData = iData;
+                    acc_xp = xp;
+                }
+                accumulate += l;
+            }
+
+            if ( l <= 0 ) {
+                /*  rh: 20040215
+                 *  may happen due to "prev_data_use" optimization 
+                 */
+                if (accumulate01) {
+                    quantize_lines_xrpow_01(accumulate01, istep, acc_xp, acc_iData);
+                    accumulate01 = 0;
+                }
+                if (accumulate) {
+                    gfc->quantize_lines_xrpow(accumulate, istep, acc_xp, acc_iData);
+                    accumulate = 0;
+                }
+
+                break;  /* ends for-loop */
+            }
+        }
+        if (sfb <= sfbmax) {
+            iData += cod_info->width[sfb];
+            xp += cod_info->width[sfb];
+            j += cod_info->width[sfb];
+        }
+    }
+    if (accumulate) { /*last data part*/
+        gfc->quantize_lines_xrpow(accumulate, istep, acc_xp, acc_iData);
+        accumulate = 0;
+    }
+    if (accumulate01) { /*last data part*/
+        quantize_lines_xrpow_01(accumulate01, istep, acc_xp, acc_iData);
+        accumulate01 = 0;
+    }
+
+}
+
+
+
+
+void quantize_init (lame_internal_flags * const gfc)
+{
+    if (gfc->quantization)
+        gfc->quantize_lines_xrpow = quantize_lines_xrpow;
+    else
+        gfc->quantize_lines_xrpow = quantize_lines_xrpow_ISO;
+}
+
+
+
+
+
 /*************************************************************************/
 /*	      ix_max							 */
 /*************************************************************************/
@@ -736,30 +719,35 @@ static int choose_table_nonMMX(
 /*************************************************************************/
 int noquant_count_bits(
           lame_internal_flags * const gfc, 
-          gr_info * const gi
-	  )
+          gr_info * const gi,
+          calc_noise_data* prev_noise)
 {
     int bits = 0;
     int i, a1, a2;
     int *const ix = gi->l3_enc;
-    i=576;
+
+    i = Min(576, ((gi->max_nonzero_coeff+2)>>1)<<1);
+
+    if (prev_noise)
+        prev_noise->sfb_count1 = 0;
+
     /* Determine count1 region */
     for (; i > 1; i -= 2) 
-	if (ix[i - 1] | ix[i - 2])
-	    break;
+	    if (ix[i - 1] | ix[i - 2])
+	        break;
     gi->count1 = i;
 
     /* Determines the number of bits to encode the quadruples. */
     a1 = a2 = 0;
     for (; i > 3; i -= 4) {
-	int p;
-	/* hack to check if all values <= 1 */
-	if ((unsigned int)(ix[i-1] | ix[i-2] | ix[i-3] | ix[i-4]) > 1)
-	    break;
+	    int p;
+	    /* hack to check if all values <= 1 */
+	    if ((unsigned int)(ix[i-1] | ix[i-2] | ix[i-3] | ix[i-4]) > 1)
+	        break;
 
-	p = ((ix[i-4] * 2 + ix[i-3]) * 2 + ix[i-2]) * 2 + ix[i-1];
-	a1 += t32l[p];
-	a2 += t33l[p];
+	    p = ((ix[i-4] * 2 + ix[i-3]) * 2 + ix[i-2]) * 2 + ix[i-1];
+	    a1 += t32l[p];
+	    a2 += t33l[p];
     }
 
     bits = a1;
@@ -772,7 +760,7 @@ int noquant_count_bits(
     gi->count1bits = bits;
     gi->big_values = i;
     if (i == 0)
-	return bits;
+	    return bits;
 
     if (gi->block_type == SHORT_TYPE) {
       a1=3*gfc->scalefac_band.s[3];
@@ -821,12 +809,24 @@ int noquant_count_bits(
 	bits = gi->part2_3_length;
     }
 
+
+    if (prev_noise) {
+        if (gi->block_type == NORM_TYPE) {
+            int line = 0;
+            int sfb = 0;
+            while (gfc->scalefac_band.l[sfb] < gi->big_values) {
+                sfb++;
+            }
+            prev_noise->sfb_count1 = sfb;
+        }
+    }
+
     return bits;
 }
 
 int count_bits(
           lame_internal_flags * const gfc, 
-    const FLOAT8  * const xr,
+    const FLOAT  * const xr,
           gr_info * const gi,
           calc_noise_data* prev_noise
 	  )
@@ -834,46 +834,22 @@ int count_bits(
     int *const ix = gi->l3_enc;
 
     /* since quantize_xrpow uses table lookup, we need to check this first: */
-    FLOAT8 w = (IXMAX_VAL) / IPOW20(gi->global_gain);
+    FLOAT w = (IXMAX_VAL) / IPOW20(gi->global_gain);
 
-    if (gfc->gfp->VBR == vbr_mtrh) {
-        /* we need to do a full check */
-        int i = 0;
-        int max_coeff = Min(gi->max_nonzero_coeff + 4, 576);
-        max_coeff = max_coeff >> 2;
-    
-        /* this small loop is consuming about 5% of total time */
-        do {
-	        if (xr[i] > w)
-	            return LARGE_BITS;
-	        if (xr[i+1] > w)
-	            return LARGE_BITS;
-	        if (xr[i+2] > w)
-	            return LARGE_BITS;
-	        if (xr[i+3] > w)
-	            return LARGE_BITS;
-            i+=4;
-        } while (--max_coeff);
+    if (gi->xrpow_max > w)
+        return LARGE_BITS;    
 
-    } else {
-        if (gi->xrpow_max > w)
-            return LARGE_BITS;
-    }
-    
-
-    if (gfc->quantization) 
-	    quantize_xrpow(xr, ix, IPOW20(gi->global_gain), gi, prev_noise);
-    else
-	    quantize_xrpow_ISO(xr, ix, IPOW20(gi->global_gain), gi, prev_noise);
+	quantize_xrpow(xr, ix, IPOW20(gi->global_gain), gi, prev_noise, gfc);
 
     if (gfc->substep_shaping & 2) {
 	int sfb, j = 0;
 	/* 0.634521682242439 = 0.5946*2**(.5*0.1875) */
-	const FLOAT8 roundfac =
+	const FLOAT roundfac =
 	    0.634521682242439 / IPOW20(gi->global_gain+gi->scalefac_scale);
 	for (sfb = 0; sfb < gi->sfbmax; sfb++) {
 	    int width = gi->width[sfb];
 	    int l;
+        assert( width >= 0 );
 	    j += width;
 	    if (!gfc->pseudohalf[sfb])
 		continue;
@@ -882,7 +858,7 @@ int count_bits(
 		    ix[j+l] = 0.0;
 	}
     }
-    return noquant_count_bits(gfc, gi);
+    return noquant_count_bits(gfc, gi, prev_noise);
 }
 /***********************************************************************
   re-calculate the best scalefac_compress using scfsi
@@ -1126,60 +1102,84 @@ void best_scalefac_store(
     /* check if l3_enc=0 */
     j = 0;
     for ( sfb = 0; sfb < gi->sfbmax; sfb++ ) {
-	int width = gi->width[sfb];
-	j += width;
-	for (l = -width; l < 0; l++)
+        int width = gi->width[sfb];
+        assert( width >= 0 );
+        j += width;
+        for (l = -width; l < 0; l++) {
 	    if (gi->l3_enc[l+j]!=0)
-		break;
-	if (l==0)
-	    gi->scalefac[sfb] = recalc = -2; /* anything goes. */
+            break;
+        }
+       	if (l==0)
+            gi->scalefac[sfb] = recalc = -2; /* anything goes. */
+            /*  only best_scalefac_store and calc_scfsi 
+             *  know--and only they should know--about the magic number -2. 
+             */
     }
 
     if (!gi->scalefac_scale && !gi->preflag) {
-	int s = 0;
-	for (sfb = 0; sfb < gi->sfbmax; sfb++)
-	    if (gi->scalefac[sfb] > 0)
-		s |= gi->scalefac[sfb];
+    int s = 0;
+    for (sfb = 0; sfb < gi->sfbmax; sfb++)
+        if (gi->scalefac[sfb] > 0)
+            s |= gi->scalefac[sfb];
 
-	if (!(s & 1) && s != 0) {
-	    for (sfb = 0; sfb < gi->sfbmax; sfb++)
-		if (gi->scalefac[sfb] > 0)
-		    gi->scalefac[sfb] >>= 1;
-
-	    gi->scalefac_scale = recalc = 1;
-	}
+        if (!(s & 1) && s != 0) {
+            for (sfb = 0; sfb < gi->sfbmax; sfb++)
+                if (gi->scalefac[sfb] > 0)
+                    gi->scalefac[sfb] >>= 1;
+                    
+            gi->scalefac_scale = recalc = 1;
+        }
     }
 
     if (!gi->preflag && gi->block_type != SHORT_TYPE && gfc->mode_gr==2) {
-	for (sfb = 11; sfb < SBPSY_l; sfb++)
-	    if (gi->scalefac[sfb] < pretab[sfb] && gi->scalefac[sfb] != -2)
-		break;
-	if (sfb == SBPSY_l) {
-	    for (sfb = 11; sfb < SBPSY_l; sfb++)
-		if (gi->scalefac[sfb] > 0)
-		    gi->scalefac[sfb] -= pretab[sfb];
+        for (sfb = 11; sfb < SBPSY_l; sfb++)
+            if (gi->scalefac[sfb] < pretab[sfb] && gi->scalefac[sfb] != -2)
+            	break;
+        if (sfb == SBPSY_l) {
+            for (sfb = 11; sfb < SBPSY_l; sfb++)
+                if (gi->scalefac[sfb] > 0)
+                    gi->scalefac[sfb] -= pretab[sfb];
 
-	    gi->preflag = recalc = 1;
-	}
+            gi->preflag = recalc = 1;
+        }
     }
 
     for ( i = 0; i < 4; i++ )
-	l3_side->scfsi[ch][i] = 0;
+        l3_side->scfsi[ch][i] = 0;
 
     if (gfc->mode_gr==2 && gr == 1
-	&& l3_side->tt[0][ch].block_type != SHORT_TYPE
-	&& l3_side->tt[1][ch].block_type != SHORT_TYPE) {
+      && l3_side->tt[0][ch].block_type != SHORT_TYPE
+      && l3_side->tt[1][ch].block_type != SHORT_TYPE) {
       	scfsi_calc(ch, l3_side);
-    } else if (recalc) {
-	if (gfc->mode_gr == 2) {
-	    scale_bitcount(gi);
-	} else {
-	    scale_bitcount_lsf(gfc, gi);
-	}
+        recalc = 0;
+    }    
+    for ( sfb = 0; sfb < gi->sfbmax; sfb++ ) {
+        if ( gi->scalefac[sfb] == -2 ) {
+            gi->scalefac[sfb] = 0;  /* if anything goes, then 0 is a good choice */
+        }
+    }     
+    if (recalc) {    
+        if (gfc->mode_gr == 2) {
+            scale_bitcount(gi);
+        } else {
+            scale_bitcount_lsf(gfc, gi);
+        }
     }
 }
 
 
+#ifndef NDEBUG
+static int all_scalefactors_not_negative( int const* scalefac, int n )
+{
+    int i;
+    for ( i = 0; i < n; ++i ) {
+        if ( scalefac[i] < 0 ) return 0;
+    }
+    return 1;
+}
+#endif 
+
+
 /* number of bits used to encode scalefacs */
 
 /* 18*slen1_tab[i] + 18*slen2_tab[i] */
@@ -1208,7 +1208,9 @@ int scale_bitcount(gr_info * const cod_info)
     /* maximum values */
     const int *tab;
     int *scalefac = cod_info->scalefac;
-
+    
+    assert( all_scalefactors_not_negative( scalefac, cod_info->sfbmax ) );
+    
     if ( cod_info->block_type == SHORT_TYPE ) {
 	tab = scale_short;
 	if (cod_info->mixed_block_flag)
