@@ -1,4 +1,4 @@
-/* $Id: common.c,v 1.30 2004/01/10 10:27:28 takehiro Exp $ */
+/* $Id: common.c,v 1.31 2004/04/14 22:15:43 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -44,10 +44,6 @@ const long freqs[9] = { 44100, 48000, 32000,
                         22050, 24000, 16000,
                         11025, 12000,  8000 };
 
-int bitindex;
-unsigned char *wordpointer;
-unsigned char *pcm_sample;
-int pcm_point = 0;
 
 
 #if defined( USE_LAYER_1 ) || defined ( USE_LAYER_2 )
@@ -257,48 +253,48 @@ void print_header_compact(struct frame *fr)
 
 #endif
 
-unsigned int getbits(int number_of_bits)
+unsigned int getbits(PMPSTR mp, int number_of_bits)
 {
   unsigned long rval;
 
-  if (number_of_bits <= 0 || !wordpointer)
+  if (number_of_bits <= 0 || !mp->wordpointer)
     return 0;
 
   {
-    rval = wordpointer[0];
+    rval = mp->wordpointer[0];
     rval <<= 8;
-    rval |= wordpointer[1];
+    rval |= mp->wordpointer[1];
     rval <<= 8;
-    rval |= wordpointer[2];
-    rval <<= bitindex;
+    rval |= mp->wordpointer[2];
+    rval <<= mp->bitindex;
     rval &= 0xffffff;
 
-    bitindex += number_of_bits;
+    mp->bitindex += number_of_bits;
 
     rval >>= (24-number_of_bits);
 
-    wordpointer += (bitindex>>3);
-    bitindex &= 7;
+    mp->wordpointer += (mp->bitindex>>3);
+    mp->bitindex &= 7;
   }
   return rval;
 }
 
-unsigned int getbits_fast(int number_of_bits)
+unsigned int getbits_fast(PMPSTR mp, int number_of_bits)
 {
   unsigned long rval;
 
   {
-    rval = wordpointer[0];
+    rval = mp->wordpointer[0];
     rval <<= 8;	
-    rval |= wordpointer[1];
-    rval <<= bitindex;
+    rval |= mp->wordpointer[1];
+    rval <<= mp->bitindex;
     rval &= 0xffff;
-    bitindex += number_of_bits;
+    mp->bitindex += number_of_bits;
 
     rval >>= (16-number_of_bits);
 
-    wordpointer += (bitindex>>3);
-    bitindex &= 7;
+    mp->wordpointer += (mp->bitindex>>3);
+    mp->bitindex &= 7;
   }
   return rval;
 }
@@ -313,10 +309,10 @@ int set_pointer( PMPSTR mp, long backstep)
     return MP3_ERR; 
   }
   bsbufold = mp->bsspace[1-mp->bsnum] + 512;
-  wordpointer -= backstep;
+  mp->wordpointer -= backstep;
   if (backstep)
-    memcpy(wordpointer,bsbufold+mp->fsizeold-backstep,(size_t)backstep);
-  bitindex = 0;
+    memcpy(mp->wordpointer,bsbufold+mp->fsizeold-backstep,(size_t)backstep);
+  mp->bitindex = 0;
   return MP3_OK;
 }
 
