@@ -31,8 +31,6 @@ struct al_table2
 
 
 void init_layer2(void);
-void II_step_one(unsigned int *bit_alloc,int *scale,struct frame *fr);
-void II_step_two(unsigned int *bit_alloc,real fraction[2][4][SBLIMIT],int *scale,struct frame *fr,int x1);
 int  do_layer2( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point);
 
 #endif
