@@ -5,7 +5,7 @@
  * All rights reserved. See also 'README'
  */ 
 
-/* $Id: layer3.c,v 1.47 2004/01/10 10:27:28 takehiro Exp $ */
+/* $Id: layer3.c,v 1.48 2004/04/14 22:15:44 robert Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -21,7 +21,6 @@
 #include <dmalloc.h>
 #endif
 
-#define MPEG1
 
 
 static real ispow[8207];
@@ -42,8 +41,8 @@ struct bandInfoStruct {
   short shortDiff[13];
 };
 
-int longLimit[9][23];
-int shortLimit[9][14];
+static int longLimit[9][23];
+static int shortLimit[9][14];
 
 const struct bandInfoStruct bandInfo[9] = { 
 
@@ -105,14 +104,14 @@ static unsigned int i_slen2[256]; /* MPEG 2.0 slen for intensity stereo */
 static real tan1_1[16],tan2_1[16],tan1_2[16],tan2_2[16];
 static real pow1_1[2][16],pow2_1[2][16],pow1_2[2][16],pow2_2[2][16];
 
-static unsigned int get1bit(void)
+static unsigned int get1bit(PMPSTR mp)
 {
   unsigned char rval;
-  rval = *wordpointer << bitindex;
+  rval = *mp->wordpointer << mp->bitindex;
 
-  bitindex++;
-  wordpointer += (bitindex>>3);
-  bitindex &= 7;
+  mp->bitindex++;
+  mp->wordpointer += (mp->bitindex>>3);
+  mp->bitindex &= 7;
 
   return rval>>7;
 }
@@ -318,38 +317,38 @@ void init_layer3(int down_sample_sblimit)
 /*
  * read additional side information
  */
-#ifdef MPEG1 
-static void III_get_side_info_1(struct III_sideinfo *si,int stereo,
+
+static void III_get_side_info_1(PMPSTR mp, struct III_sideinfo *si,int stereo,
  int ms_stereo,long sfreq,int single)
 {
    int ch, gr;
    int powdiff = (single == 3) ? 4 : 0;
 
-   si->main_data_begin = getbits(9);
+   si->main_data_begin = getbits(mp,9);
    if (stereo == 1)
-     si->private_bits = getbits_fast(5);
+     si->private_bits = getbits_fast(mp,5);
    else 
-     si->private_bits = getbits_fast(3);
+     si->private_bits = getbits_fast(mp,3);
 
    for (ch=0; ch<stereo; ch++) {
        si->ch[ch].gr[0].scfsi = -1;
-       si->ch[ch].gr[1].scfsi = getbits_fast(4);
+       si->ch[ch].gr[1].scfsi = getbits_fast(mp,4);
    }
 
    for (gr=0; gr<2; gr++) 
    {
      for (ch=0; ch<stereo; ch++) 
      {
-       register struct gr_info_s *gr_infos = &(si->ch[ch].gr[gr]);
+       struct gr_info_s *gr_infos = &(si->ch[ch].gr[gr]);
 
-       gr_infos->part2_3_length = getbits(12);
-       gr_infos->big_values = getbits_fast(9);
+       gr_infos->part2_3_length = getbits(mp,12);
+       gr_infos->big_values = getbits_fast(mp,9);
        if(gr_infos->big_values > 288) {
           fprintf(stderr,"big_values too large! %i\n",gr_infos->big_values);
           gr_infos->big_values = 288;
        }
        {
-	 unsigned int qss = getbits_fast(8);
+	 unsigned int qss = getbits_fast(mp,8);
 	 gr_infos->pow2gain = gainpow2+256 - qss + powdiff;
 #ifndef NOANALYSIS
 	 if (mpg123_pinfo != NULL) {
@@ -359,15 +358,15 @@ static void III_get_side_info_1(struct III_sideinfo *si,int stereo,
        }
        if(ms_stereo)
          gr_infos->pow2gain += 2;
-       gr_infos->scalefac_compress = getbits_fast(4);
+       gr_infos->scalefac_compress = getbits_fast(mp,4);
 /* window-switching flag == 1 for block_Type != 0 .. and block-type == 0 -> win-sw-flag = 0 */
-       if(get1bit()) 
+       if(get1bit(mp)) 
        {
          int i;
-         gr_infos->block_type = getbits_fast(2);
-         gr_infos->mixed_block_flag = get1bit();
-         gr_infos->table_select[0] = getbits_fast(5);
-         gr_infos->table_select[1] = getbits_fast(5);
+         gr_infos->block_type = getbits_fast(mp,2);
+         gr_infos->mixed_block_flag = get1bit(mp);
+         gr_infos->table_select[0] = getbits_fast(mp,5);
+         gr_infos->table_select[1] = getbits_fast(mp,5);
 
 
          /*
@@ -376,7 +375,7 @@ static void III_get_side_info_1(struct III_sideinfo *si,int stereo,
           */
          gr_infos->table_select[2] = 0;
          for(i=0;i<3;i++) {
-	   unsigned int sbg = (getbits_fast(3)<<3);
+	   unsigned int sbg = (getbits_fast(mp,3)<<3);
            gr_infos->full_gain[i] = gr_infos->pow2gain + sbg;
 #ifndef NOANALYSIS
 	   if (mpg123_pinfo != NULL)
@@ -397,50 +396,49 @@ static void III_get_side_info_1(struct III_sideinfo *si,int stereo,
        {
          int i,r0c,r1c;
          for (i=0; i<3; i++)
-           gr_infos->table_select[i] = getbits_fast(5);
-         r0c = getbits_fast(4);
-         r1c = getbits_fast(3);
+           gr_infos->table_select[i] = getbits_fast(mp,5);
+         r0c = getbits_fast(mp,4);
+         r1c = getbits_fast(mp,3);
          gr_infos->region1start = bandInfo[sfreq].longIdx[r0c+1] >> 1 ;
          gr_infos->region2start = bandInfo[sfreq].longIdx[r0c+1+r1c+1] >> 1;
          gr_infos->block_type = 0;
          gr_infos->mixed_block_flag = 0;
        }
-       gr_infos->preflag = get1bit();
-       gr_infos->scalefac_scale = get1bit();
-       gr_infos->count1table_select = get1bit();
+       gr_infos->preflag = get1bit(mp);
+       gr_infos->scalefac_scale = get1bit(mp);
+       gr_infos->count1table_select = get1bit(mp);
      }
    }
 }
-#endif
 
 /*
  * Side Info for MPEG 2.0 / LSF
  */
-static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
+static void III_get_side_info_2(PMPSTR mp, struct III_sideinfo *si,int stereo,
  int ms_stereo,long sfreq,int single)
 {
    int ch;
    int powdiff = (single == 3) ? 4 : 0;
 
-   si->main_data_begin = getbits(8);
+   si->main_data_begin = getbits(mp,8);
 
    if (stereo == 1)
-     si->private_bits = get1bit();
+     si->private_bits = get1bit(mp);
    else 
-     si->private_bits = getbits_fast(2);
+     si->private_bits = getbits_fast(mp,2);
 
    for (ch=0; ch<stereo; ch++) 
    {
-       register struct gr_info_s *gr_infos = &(si->ch[ch].gr[0]);
+       struct gr_info_s *gr_infos = &(si->ch[ch].gr[0]);
        unsigned int qss;
 
-       gr_infos->part2_3_length = getbits(12);
-       gr_infos->big_values = getbits_fast(9);
+       gr_infos->part2_3_length = getbits(mp,12);
+       gr_infos->big_values = getbits_fast(mp,9);
        if(gr_infos->big_values > 288) {
          fprintf(stderr,"big_values too large! %i\n",gr_infos->big_values);
          gr_infos->big_values = 288;
        }
-       qss=getbits_fast(8);
+       qss=getbits_fast(mp,8);
        gr_infos->pow2gain = gainpow2+256 - qss + powdiff;
 #ifndef NOANALYSIS
        if (mpg123_pinfo!=NULL) {
@@ -451,22 +449,22 @@ static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
 
        if(ms_stereo)
          gr_infos->pow2gain += 2;
-       gr_infos->scalefac_compress = getbits(9);
+       gr_infos->scalefac_compress = getbits(mp,9);
 /* window-switching flag == 1 for block_Type != 0 .. and block-type == 0 -> win-sw-flag = 0 */
-       if(get1bit()) 
+       if(get1bit(mp)) 
        {
          int i;
-         gr_infos->block_type = getbits_fast(2);
-         gr_infos->mixed_block_flag = get1bit();
-         gr_infos->table_select[0] = getbits_fast(5);
-         gr_infos->table_select[1] = getbits_fast(5);
+         gr_infos->block_type = getbits_fast(mp,2);
+         gr_infos->mixed_block_flag = get1bit(mp);
+         gr_infos->table_select[0] = getbits_fast(mp,5);
+         gr_infos->table_select[1] = getbits_fast(mp,5);
          /*
           * table_select[2] not needed, because there is no region2,
           * but to satisfy some verifications tools we set it either.
           */
          gr_infos->table_select[2] = 0;
          for(i=0;i<3;i++) {
-	   unsigned int sbg = (getbits_fast(3)<<3);
+	   unsigned int sbg = (getbits_fast(mp,3)<<3);
            gr_infos->full_gain[i] = gr_infos->pow2gain + sbg;
 #ifndef NOANALYSIS
 	   if (mpg123_pinfo!=NULL)
@@ -499,24 +497,25 @@ static void III_get_side_info_2(struct III_sideinfo *si,int stereo,
        {
          int i,r0c,r1c;
          for (i=0; i<3; i++)
-           gr_infos->table_select[i] = getbits_fast(5);
-         r0c = getbits_fast(4);
-         r1c = getbits_fast(3);
+           gr_infos->table_select[i] = getbits_fast(mp,5);
+         r0c = getbits_fast(mp,4);
+         r1c = getbits_fast(mp,3);
          gr_infos->region1start = bandInfo[sfreq].longIdx[r0c+1] >> 1 ;
          gr_infos->region2start = bandInfo[sfreq].longIdx[r0c+1+r1c+1] >> 1;
          gr_infos->block_type = 0;
          gr_infos->mixed_block_flag = 0;
        }
-       gr_infos->scalefac_scale = get1bit();
-       gr_infos->count1table_select = get1bit();
+       gr_infos->scalefac_scale = get1bit(mp);
+       gr_infos->count1table_select = get1bit(mp);
    }
 }
 
 /*
  * read scalefactors
  */
-#ifdef MPEG1
-static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
+
+static int
+III_get_scale_factors_1(PMPSTR mp, int *scf,struct gr_info_s *gr_infos)
 {
    static const unsigned char slen[2][16] = {
      {0, 0, 0, 0, 3, 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4},
@@ -533,15 +532,15 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
 
       if (gr_infos->mixed_block_flag) {
          for (i=8;i;i--)
-           *scf++ = getbits_fast(num0);
+           *scf++ = getbits_fast(mp,num0);
          i = 9;
          numbits -= num0; /* num0 * 17 + num1 * 18 */
       }
 
       for (;i;i--)
-        *scf++ = getbits_fast(num0);
+        *scf++ = getbits_fast(mp,num0);
       for (i = 18; i; i--)
-        *scf++ = getbits_fast(num1);
+        *scf++ = getbits_fast(mp,num1);
       *scf++ = 0; *scf++ = 0; *scf++ = 0; /* short[13][0..2] = 0 */
     }
     else 
@@ -551,16 +550,16 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
 
       if(scfsi < 0) { /* scfsi < 0 => granule == 0 */
          for(i=11;i;i--)
-           *scf++ = getbits_fast(num0);
+           *scf++ = getbits_fast(mp,num0);
          for(i=10;i;i--)
-           *scf++ = getbits_fast(num1);
+           *scf++ = getbits_fast(mp,num1);
          numbits = (num0 + num1) * 10 + num0;
       }
       else {
         numbits = 0;
         if(!(scfsi & 0x8)) {
           for (i=6;i;i--)
-            *scf++ = getbits_fast(num0);
+            *scf++ = getbits_fast(mp,num0);
           numbits += num0 * 6;
         }
         else {
@@ -569,7 +568,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
 
         if(!(scfsi & 0x4)) {
           for (i=5;i;i--)
-            *scf++ = getbits_fast(num0);
+            *scf++ = getbits_fast(mp,num0);
           numbits += num0 * 5;
         }
         else {
@@ -578,7 +577,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
 
         if(!(scfsi & 0x2)) {
           for(i=5;i;i--)
-            *scf++ = getbits_fast(num1);
+            *scf++ = getbits_fast(mp,num1);
           numbits += num1 * 5;
         }
         else {
@@ -587,7 +586,7 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
 
         if(!(scfsi & 0x1)) {
           for (i=5;i;i--)
-            *scf++ = getbits_fast(num1);
+            *scf++ = getbits_fast(mp,num1);
           numbits += num1 * 5;
         }
         else {
@@ -599,9 +598,10 @@ static int III_get_scale_factors_1(int *scf,struct gr_info_s *gr_infos)
     }
     return numbits;
 }
-#endif
 
-static int III_get_scale_factors_2(int *scf,struct gr_info_s *gr_infos,int i_stereo)
+
+static int
+III_get_scale_factors_2(PMPSTR mp, int *scf,struct gr_info_s *gr_infos,int i_stereo)
 {
   unsigned char *pnt;
   int i,j;
@@ -638,7 +638,7 @@ static int III_get_scale_factors_2(int *scf,struct gr_info_s *gr_infos,int i_ste
     slen >>= 3;
     if(num) {
       for(j=0;j<(int)(pnt[i]);j++)
-        *scf++ = getbits_fast(num);
+        *scf++ = getbits_fast(mp,num);
       numbits += pnt[i] * num;
     }
     else {
@@ -660,7 +660,7 @@ static const int pretab2 [22] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
 /*
  * don't forget to apply the same changes to III_dequantize_sample_ms() !!! 
  */
-static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
+static int III_dequantize_sample(PMPSTR mp, real xr[SBLIMIT][SSLIMIT],int *scf,
    struct gr_info_s *gr_infos,int sfreq,int part2bits)
 {
   int shift = 1 + gr_infos->scalefac_scale;
@@ -721,8 +721,8 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
      */
     int i,max[4];
     int step=0,lwin=0,cb=0;
-    register real v = 0.0;
-    register int *m,mc;
+    real v = 0.0;
+    int *m,mc;
 
     if(gr_infos->mixed_block_flag) {
       max[3] = -1;
@@ -742,7 +742,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
       int lp = l[i];
       struct newhuff *h = (struct newhuff *)(ht+gr_infos->table_select[i]);
       for(;lp;lp--,mc--) {
-        register int x,y;
+        int x,y;
         if( (!mc) ) {
           mc = *m++;
           xrpnt = ((real *) xr) + (*m++);
@@ -758,9 +758,9 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
           }
         }
         {
-          register short *val = (short *)h->table;
+          short *val = (short *)h->table;
           while((y=*val++)<0) {
-            if (get1bit())
+            if (get1bit(mp))
               val -= y;
             part2remain--;
           }
@@ -770,15 +770,15 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
         if(x == 15) {
           max[lwin] = cb;
           part2remain -= h->linbits+1;
-          x += getbits((int)h->linbits);
-          if(get1bit())
+          x += getbits(mp,(int)h->linbits);
+          if(get1bit(mp))
             *xrpnt = -ispow[x] * v;
           else
             *xrpnt =  ispow[x] * v;
         }
         else if(x) {
           max[lwin] = cb;
-          if(get1bit())
+          if(get1bit(mp))
             *xrpnt = -ispow[x] * v;
           else
             *xrpnt =  ispow[x] * v;
@@ -790,15 +790,15 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
         if(y == 15) {
           max[lwin] = cb;
           part2remain -= h->linbits+1;
-          y += getbits((int)h->linbits);
-          if(get1bit())
+          y += getbits(mp,(int)h->linbits);
+          if(get1bit(mp))
             *xrpnt = -ispow[y] * v;
           else
             *xrpnt =  ispow[y] * v;
         }
         else if(y) {
           max[lwin] = cb;
-          if(get1bit())
+          if(get1bit(mp))
             *xrpnt = -ispow[y] * v;
           else
             *xrpnt =  ispow[y] * v;
@@ -811,7 +811,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
     }
     for(;l3 && (part2remain > 0);l3--) {
       struct newhuff *h = (struct newhuff *)(htc+gr_infos->count1table_select);
-      register short *val = (short *)h->table,a;
+      short *val = (short *)h->table,a;
 
       while((a=*val++)<0) {
         part2remain--;
@@ -820,7 +820,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
           a = 0;
           break;
         }
-        if (get1bit())
+        if (get1bit(mp))
           val -= a;
       }
       for(i=0;i<4;i++) {
@@ -848,7 +848,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
             part2remain++;
             break;
           }
-          if(get1bit()) 
+          if(get1bit(mp)) 
             *xrpnt = -v;
           else
             *xrpnt = v;
@@ -900,12 +900,9 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
     int *pretab = (int *)(gr_infos->preflag ? pretab1 : pretab2);
     int i,max = -1;
     int cb = 0;
-    register int *m = map[sfreq][2];
-    register real v = 0.0;
-    register int mc = 0;
-#if 0
-    me = mapend[sfreq][2];
-#endif
+    int *m = map[sfreq][2];
+    real v = 0.0;
+    int mc = 0;
 
 	/*
      * long hash table values
@@ -923,9 +920,9 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
           cb = *m++;
         }
         {
-          register short *val = (short *)h->table;
+          short *val = (short *)h->table;
           while((y=*val++)<0) {
-            if (get1bit())
+            if (get1bit(mp))
               val -= y;
             part2remain--;
           }
@@ -935,15 +932,15 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
         if (x == 15) {
           max = cb;
           part2remain -= h->linbits+1;
-          x += getbits((int)h->linbits);
-          if(get1bit())
+          x += getbits(mp,(int)h->linbits);
+          if(get1bit(mp))
             *xrpnt++ = -ispow[x] * v;
           else
             *xrpnt++ =  ispow[x] * v;
         }
         else if(x) {
           max = cb;
-          if(get1bit())
+          if(get1bit(mp))
             *xrpnt++ = -ispow[x] * v;
           else
             *xrpnt++ =  ispow[x] * v;
@@ -955,15 +952,15 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
         if (y == 15) {
           max = cb;
           part2remain -= h->linbits+1;
-          y += getbits((int)h->linbits);
-          if(get1bit())
+          y += getbits(mp,(int)h->linbits);
+          if(get1bit(mp))
             *xrpnt++ = -ispow[y] * v;
           else
             *xrpnt++ =  ispow[y] * v;
         }
         else if(y) {
           max = cb;
-          if(get1bit())
+          if(get1bit(mp))
             *xrpnt++ = -ispow[y] * v;
           else
             *xrpnt++ =  ispow[y] * v;
@@ -979,7 +976,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
      */
     for(;l3 && (part2remain > 0);l3--) {
       struct newhuff *h = (struct newhuff *)(htc+gr_infos->count1table_select);
-      register short *val = (short *)h->table,a;
+      short *val = (short *)h->table,a;
 
       while((a=*val++)<0) {
         part2remain--;
@@ -988,7 +985,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
           a = 0;
           break;
         }
-        if (get1bit())
+        if (get1bit(mp))
           val -= a;
       }
       for(i=0;i<4;i++) {
@@ -1007,7 +1004,7 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
             part2remain++;
             break;
           }
-          if(get1bit())
+          if(get1bit(mp))
             *xrpnt++ = -v;
           else
             *xrpnt++ = v;
@@ -1030,11 +1027,11 @@ static int III_dequantize_sample(real xr[SBLIMIT][SSLIMIT],int *scf,
   }
 
   while( part2remain > 16 ) {
-    getbits(16); /* Dismiss stuffing Bits */
+    getbits(mp,16); /* Dismiss stuffing Bits */
     part2remain -= 16;
   }
   if(part2remain > 0)
-    getbits(part2remain);
+    getbits(mp,part2remain);
   else if(part2remain < 0) {
     fprintf(stderr,"mpg123: Can't rewind stream by %d bits!\n",-part2remain);
     return 1; /* -> error */
@@ -1219,7 +1216,7 @@ static void III_antialias(real xr[SBLIMIT][SSLIMIT],struct gr_info_s *gr_infos)
 
        for(ss=7;ss>=0;ss--)
        {       /* upper and lower butterfly inputs */
-         register real bu = *--xr2,bd = *xr1;
+         real bu = *--xr2,bd = *xr1;
          *xr2   = (bu * (*cs)   ) - (bd * (*ca)   );
          *xr1++ = (bd * (*cs++) ) + (bu * (*ca++) );
        }
@@ -1240,7 +1237,7 @@ static void III_antialias(real xr[SBLIMIT][SSLIMIT],struct gr_info_s *gr_infos)
 static void dct36(real *inbuf,real *o1,real *o2,real *wintab,real *tsbuf)
 {
   {
-    register real *in = inbuf;
+    real *in = inbuf;
 
     in[17]+=in[16]; in[16]+=in[15]; in[15]+=in[14];
     in[14]+=in[13]; in[13]+=in[12]; in[12]+=in[11];
@@ -1273,11 +1270,11 @@ static void dct36(real *inbuf,real *o1,real *o2,real *wintab,real *tsbuf)
     sum1 = (tmp2b - tmp1b) * tfcos36[(v)]; \
 	MACRO0(v); }
 
-    register const real *c = COS9;
-    register real *out2 = o2;
-	register real *w = wintab;
-	register real *out1 = o1;
-	register real *ts = tsbuf;
+    const real *c = COS9;
+    real *out2 = o2;
+	real *w = wintab;
+	real *out1 = o1;
+	real *ts = tsbuf;
 
     real ta33,ta66,tb33,tb66;
 
@@ -1344,7 +1341,7 @@ static void dct36(real *inbuf,real *o1,real *o2,real *wintab,real *tsbuf)
 /*
  * new DCT12
  */
-static void dct12(real *in,real *rawout1,real *rawout2,register real *wi,register real *ts)
+static void dct12(real *in,real *rawout1,real *rawout2,real *wi,real *ts)
 {
 #define DCT12_PART1 \
              in5 = in[5*3];  \
@@ -1379,7 +1376,7 @@ static void dct12(real *in,real *rawout1,real *rawout2,register real *wi,registe
 
    {
      real in0,in1,in2,in3,in4,in5;
-     register real *out1 = rawout1;
+     real *out1 = rawout1;
      ts[SBLIMIT*0] = out1[0]; ts[SBLIMIT*1] = out1[1]; ts[SBLIMIT*2] = out1[2];
      ts[SBLIMIT*3] = out1[3]; ts[SBLIMIT*4] = out1[4]; ts[SBLIMIT*5] = out1[5];
  
@@ -1415,7 +1412,7 @@ static void dct12(real *in,real *rawout1,real *rawout2,register real *wi,registe
 
   {
      real in0,in1,in2,in3,in4,in5;
-     register real *out2 = rawout2;
+     real *out2 = rawout2;
  
      DCT12_PART1
 
@@ -1449,7 +1446,7 @@ static void dct12(real *in,real *rawout1,real *rawout2,register real *wi,registe
 
   {
      real in0,in1,in2,in3,in4,in5;
-     register real *out2 = rawout2;
+     real *out2 = rawout2;
      out2[12]=out2[13]=out2[14]=out2[15]=out2[16]=out2[17]=0.0;
 
      DCT12_PART1
@@ -1538,8 +1535,9 @@ static void III_hybrid( PMPSTR mp, real fsIn[SBLIMIT][SSLIMIT],real tsOut[SSLIMI
  */
 struct III_sideinfo sideinfo;
 
-int do_layer3_sideinfo(struct frame *fr)
+int do_layer3_sideinfo(PMPSTR mp)
 {
+  struct frame *fr = &mp->fr;
   int stereo = fr->stereo;
   int single = fr->single;
   int ms_stereo;
@@ -1560,15 +1558,11 @@ int do_layer3_sideinfo(struct frame *fr)
 
   if(fr->lsf) {
     granules = 1;
-    III_get_side_info_2(&sideinfo,stereo,ms_stereo,sfreq,single);
+    III_get_side_info_2(mp,&sideinfo,stereo,ms_stereo,sfreq,single);
   }
   else {
     granules = 2;
-#ifdef MPEG1
-    III_get_side_info_1(&sideinfo,stereo,ms_stereo,sfreq,single);
-#else
-    fprintf(stderr,"Not supported\n");
-#endif
+    III_get_side_info_1(mp,&sideinfo,stereo,ms_stereo,sfreq,single);
   }
 
   databits=0;
@@ -1635,13 +1629,9 @@ int  do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point,
       long part2bits;
 
       if(fr->lsf)
-        part2bits = III_get_scale_factors_2(scalefacs[0],gr_infos,0);
+        part2bits = III_get_scale_factors_2(mp,scalefacs[0],gr_infos,0);
       else {
-#ifdef MPEG1
-        part2bits = III_get_scale_factors_1(scalefacs[0],gr_infos);
-#else
-	fprintf(stderr,"Not supported\n");
-#endif
+        part2bits = III_get_scale_factors_1(mp,scalefacs[0],gr_infos);
       }
 
 #ifndef NOANALYSIS
@@ -1653,20 +1643,16 @@ int  do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point,
       }
 #endif
 
-      if(III_dequantize_sample(hybridIn[0], scalefacs[0],gr_infos,sfreq,part2bits))
+      if(III_dequantize_sample(mp, hybridIn[0], scalefacs[0],gr_infos,sfreq,part2bits))
         return clip;
     }
     if(stereo == 2) {
       struct gr_info_s *gr_infos = &(sideinfo.ch[1].gr[gr]);
       long part2bits;
       if(fr->lsf) 
-        part2bits = III_get_scale_factors_2(scalefacs[1],gr_infos,i_stereo);
+        part2bits = III_get_scale_factors_2(mp,scalefacs[1],gr_infos,i_stereo);
       else {
-#ifdef MPEG1
-        part2bits = III_get_scale_factors_1(scalefacs[1],gr_infos);
-#else
-	fprintf(stderr,"Not supported\n");
-#endif
+        part2bits = III_get_scale_factors_1(mp,scalefacs[1],gr_infos);
       }
 #ifndef NOANALYSIS
       if (mpg123_pinfo!=NULL) {
@@ -1677,7 +1663,7 @@ int  do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point,
       }
 #endif
 
-      if(III_dequantize_sample(hybridIn[1],scalefacs[1],gr_infos,sfreq,part2bits))
+      if(III_dequantize_sample(mp, hybridIn[1],scalefacs[1],gr_infos,sfreq,part2bits))
           return clip;
 
       if(ms_stereo) {
@@ -1704,16 +1690,16 @@ int  do_layer3( PMPSTR mp,unsigned char *pcm_sample,int *pcm_point,
       switch(single) {
         case 3:
           {
-            register int i;
-            register real *in0 = (real *) hybridIn[0],*in1 = (real *) hybridIn[1];
+            int i;
+            real *in0 = (real *) hybridIn[0],*in1 = (real *) hybridIn[1];
             for(i=0;i<(int)(SSLIMIT*gr_infos->maxb);i++,in0++)
               *in0 = (*in0 + *in1++); /* *0.5 done by pow-scale */ 
           }
           break;
         case 1:
           {
-            register int i;
-            register real *in0 = (real *) hybridIn[0],*in1 = (real *) hybridIn[1];
+            int i;
+            real *in0 = (real *) hybridIn[0],*in1 = (real *) hybridIn[1];
             for(i=0;i<(int)(SSLIMIT*gr_infos->maxb);i++)
               *in0++ = *in1++;
           }
